import doctest
from insights.combiners import cloud_provider
from insights.combiners.cloud_provider import CloudProvider
from insights.parsers.installed_rpms import InstalledRpms as IRPMS
from insights.parsers.dmidecode import DMIDecode
from insights.parsers.rhsm_conf import RHSMConf
from insights.parsers.yum import YumRepoList
from insights.tests import context_wrap

RPMS = """
gnome-terminal-3.28.2-2.fc28.x86_64
python3-IPy-0.81-21.fc28.noarch
gnu-free-serif-fonts-20120503-17.fc28.noarch
""".strip()

RPMS_AWS = """
gnome-terminal-3.28.2-2.fc28.x86_64
python3-IPy-0.81-21.fc28.noarch
gnu-free-serif-fonts-20120503-17.fc28.noarch
rh-amazon-rhui-client-2.2.124-1.el7
""".strip()

RPMS_GOOGLE = """
gnome-terminal-3.28.2-2.fc28.x86_64
python3-IPy-0.81-21.fc28.noarch
gnu-free-serif-fonts-20120503-17.fc28.noarch
google-rhui-client-5.1.100-1.el7
google-rhui-client-5.1.100-1.el6
""".strip()

RPMS_AZURE = """
gnome-terminal-3.28.2-2.fc28.x86_64
python3-IPy-0.81-21.fc28.noarch
gnu-free-serif-fonts-20120503-17.fc28.noarch
WALinuxAgent-2.2.18-1.el7
""".strip()

YUM_REPOLIST_AZURE = """
Loaded plugins: enabled_repos_upload, package_upload, product-id, search-
              : disabled-repos, security, subscription-manager
repo id                                repo name                          status
rhel-6-server-rpms                     Red Hat Enterprise Linux 6 Server  20584
rhel-6-server-satellite-tools-6.3-rpms Red Hat Satellite Tools 6.3 (for R    40
rhui-microsoft-azure-rhel7-2.2-74      Red Hat Software Collect             600
repolist: 21224
Uploading Enabled Repositories Report
Loaded plugins: product-id, subscription-manager
""".strip()

YUM_REPOLIST_NOT_AZURE = """
Loaded plugins: enabled_repos_upload, package_upload, product-id, search-
              : disabled-repos, security, subscription-manager
repo id                                repo name                          status
rhel-6-server-rpms                     Red Hat Enterprise Linux 6 Server  20584
rhel-6-server-satellite-tools-6.3-rpms Red Hat Satellite Tools 6.3 (for R    40
repolist: 21224
Uploading Enabled Repositories Report
Loaded plugins: product-id, subscription-manager
""".strip()


DMIDECODE = '''
# dmidecode 2.11
SMBIOS 2.7 present.
188 structures occupying 5463 bytes.
Table at 0xBFFCB000.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: HP
\tVersion: P70
\tRelease Date: 03/01/2013
\tAddress: 0xF0000
\tRuntime Size: 64 kB
\tROM Size: 8192 kB
\tCharacteristics:
\t\tPCI is supported
\t\tPNP is supported
\t\tBIOS is upgradeable
\t\tBIOS shadowing is allowed
\t\tESCD support is available
\t\tBoot from CD is supported
\t\tSelectable boot is supported
\t\tEDD is supported
\t\t5.25"/360 kB floppy services are supported (int 13h)
\t\t5.25"/1.2 MB floppy services are supported (int 13h)
\t\t3.5"/720 kB floppy services are supported (int 13h)
\t\tPrint screen service is supported (int 5h)
\t\tj042 keyboard services are supported (int 9h)
\t\tSerial services are supported (int 14h)
\t\tPrinter services are supported (int 17h)
\t\tCGA/mono video services are supported (int 10h)
\t\tACPI is supported
\t\tUSB legacy is supported
\t\tBIOS boot specification is supported
\t\tFunction key-initiated network boot is supported
\t\tTargeted content distribution is supported
\tFirmware Revision: 1.22

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: HP
\tProduct Name: ProLiant DL380p Gen8
\tVersion: Not Specified
\tSerial Number: 2M23360006
\tUUID: 34373936-3439-4D32-3233-333630303036
\tWake-up Type: Power Switch
\tSKU Number: 697494-S01
\tFamily: ProLiant

Handle 0x0300, DMI type 3, 21 bytes
chassis information
\tmanufacturer: hp
\ttype: rack mount chassis
\tlock: not present
\tversion: not specified
\tserial number: 2m23360006
\tasset Tag:
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
\tOEM Information: 0x00000000
\tHeight: 2 U
\tNumber Of Power Cords: 2
\tContained Elements: 0

Handle 0x0401, DMI type 4, 32 bytes
Processor Information
\tSocket Designation: CPU 1
\tType: Central Processor
\tFamily: Other
\tManufacturer: Bochs
\tID: A1 06 02 00 FD FB 8B 17
\tVersion: Not Specified
\tVoltage: Unknown
\tExternal Clock: Unknown
\tMax Speed: 2000 MHz
\tCurrent Speed: 2000 MHz
\tStatus: Populated, Enabled
\tUpgrade: Other
\tL1 Cache Handle: Not Provided
\tL2 Cache Handle: Not Provided
\tL3 Cache Handle: Not Provided

Handle 0x0402, DMI type 4, 32 bytes
Processor Information
\tSocket Designation: CPU 2
\tType: Central Processor
\tFamily: Other
\tManufacturer: Bochs
\tID: A1 06 02 00 FD FB 8B 17
\tVersion: Not Specified
\tVoltage: Unknown
\tExternal Clock: Unknown
\tMax Speed: 2000 MHz
\tCurrent Speed: 2000 MHz
\tStatus: Populated, Enabled
\tUpgrade: Other
\tL1 Cache Handle: Not Provided
\tL2 Cache Handle: Not Provided
\tL3 Cache Handle: Not Provided

Handle 0x0037, DMI type 127, 4 bytes.
End Of Table
'''

DMIDECODE_BARE_METAL = '''
# dmidecode 3.0
Getting SMBIOS data from sysfs.
SMBIOS 2.7 present.
56 structures occupying 2407 bytes.
Table at 0x000EB4C0.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: American Megatrends Inc.
\tVersion: 2.0a
\tRelease Date: 06/08/2012
\tAddress: 0xF0000
\tRuntime Size: 64 kB
\tROM Size: 8192 kB
\tCharacteristics:
\t\tPCI is supported
\t\tBIOS is upgradeable
\t\tBIOS shadowing is allowed
\t\tBoot from CD is supported
\t\tSelectable boot is supported
\t\tBIOS ROM is socketed
\t\tEDD is supported
\t\t5.25"/1.2 MB floppy services are supported (int 13h)
\t\t3.5"/720 kB floppy services are supported (int 13h)
\t\t3.5"/2.88 MB floppy services are supported (int 13h)
\t\tPrint screen service is supported (int 5h)
\t\t8042 keyboard services are supported (int 9h)
\t\tSerial services are supported (int 14h)
\t\tPrinter services are supported (int 17h)
\t\tACPI is supported
\t\tUSB legacy is supported
\t\tBIOS boot specification is supported
\t\tFunction key-initiated network boot is supported
\t\tTargeted content distribution is supported
\t\tUEFI is supported
\tBIOS Revision: 2.10

Handle 0x0001, DMI type 1, 27 bytes
System Information
\tManufacturer: Supermicro
\tProduct Name: X9SCL/X9SCM
\tVersion: 0123456789
\tSerial Number: 0123456789
\tUUID: 12345678-1234-1234-1234-123456681234
\tWake-up Type: Power Switch
\tSKU Number: To be filled by O.E.M.
\tFamily: To be filled by O.E.M.

Handle 0x0002, DMI type 2, 15 bytes
Base Board Information
\tManufacturer: Supermicro
\tProduct Name: X9SCL/X9SCM
\tVersion: 0123456789
\tSerial Number: 1234567812
\tAsset Tag: To be filled by O.E.M.
\tFeatures:
\t\tBoard is a hosting board
\t\tBoard is replaceable
\tLocation In Chassis: To be filled by O.E.M.
\tChassis Handle: 0x0003
\tType: Motherboard
\tContained Object Handles: 0
'''.strip()

DMIDECODE_AWS = '''
# dmidecode 2.12-dmifs
SMBIOS 2.4 present.
11 structures occupying 310 bytes.
Table at 0x000EB01F.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: Xen
\tVersion: 4.2.amazon
\tRelease Date: 12/09/2016
\tAddress: 0xE8000
\tRuntime Size: 96 kB
\tROM Size: 64 kB
\tCharacteristics:
\t\tPCI is supported
\t\tEDD is supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 4.2

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: Xen
\tProduct Name: HVM domU
\tVersion: 4.2.amazon
\tSerial Number: ec2f58af-2dad-c57e-88c0-a81cb6084290
\tUUID: EC2F58AF-2DAD-C57E-88C0-A81CB6084290
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Not Specified

Handle 0x0300, DMI type 3, 13 bytes
Chassis Information
\tManufacturer: Xen
\tType: Other
\tLock: Not Present
\tVersion: Not Specified
\tSerial Number: Not Specified
\tAsset Tag: Not Specified
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
'''

DMIDECODE_AWS_UUID = '''
# dmidecode 2.12-dmifs
SMBIOS 2.4 present.
11 structures occupying 310 bytes.
Table at 0x000EB01F.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: Xen
\tVersion: 4.2
\tRelease Date: 12/09/2016
\tAddress: 0xE8000
\tRuntime Size: 96 kB
\tROM Size: 64 kB
\tCharacteristics:
\t\tPCI is supported
\t\tEDD is supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 4.2

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: Xen
\tProduct Name: HVM domU
\tVersion: 4.2.amazon
\tSerial Number: ec2f58af-2dad-c57e-88c0-a81cb6084290
\tUUID: EC2F58AF-2DAD-C57E-88C0-A81CB6084290
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Not Specified

Handle 0x0300, DMI type 3, 13 bytes
Chassis Information
\tManufacturer: Xen
\tType: Other
\tLock: Not Present
\tVersion: Not Specified
\tSerial Number: Not Specified
\tAsset Tag: Not Specified
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
'''

DMIDECODE_GOOGLE = '''
# dmidecode 2.12-dmifs
SMBIOS 2.4 present.
11 structures occupying 310 bytes.
Table at 0x000EB01F.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: Google
\tVersion: Google
\tRelease Date: 12/09/2016
\tAddress: 0xE8000
\tRuntime Size: 96 kB
\tROM Size: 64 kB
\tCharacteristics:
\t\tPCI is supported
\t\tEDD is supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 4.2

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: Xen
\tProduct Name: HVM domU
\tVersion: 4.2.amazon
\tSerial Number: ec2f58af-2dad-c57e-88c0-a81cb6084290
\tUUID: EC2F58AF-2DAD-C57E-88C0-A81CB6084290
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Not Specified

Handle 0x0300, DMI type 3, 13 bytes
Chassis Information
\tManufacturer: Xen
\tType: Other
\tLock: Not Present
\tVersion: Not Specified
\tSerial Number: Not Specified
\tAsset Tag: Not Specified
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
'''

DMIDECODE_AZURE_ASSET_TAG = """
# dmidecode 3.1
Getting SMBIOS data from sysfs.
SMBIOS 2.8 present.
10 structures occupying 511 bytes.
Table at 0x000F6050.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: SeaBIOS
\tVersion: 1.11.0-2.el7
\tRelease Date: 04/01/2014
\tAddress: 0xE8000
\tRuntime Size: 96 kB
\tROM Size: 64 kB
\tCharacteristics:
\t\tBIOS characteristics not supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 0.0

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: oVirt
\tProduct Name: oVirt Node
\tVersion: 7-5.1804.4.el7.centos
\tSerial Number: 30393137-3436-584D-5136-323830304E46
\tUUID: a35ae32b-ed0a-49a4-9dbb-eecf21f88aab
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Red Hat Enterprise Linux

Handle 0x0300, DMI type 3, 21 bytes
Chassis Information
\tManufacturer: Red Hat
\tType: Other
\tLock: Not Present
\tVersion: RHEL 7.2.0 PC (i440FX + PIIX, 1996)
\tSerial Number: Not Specified
\tAsset Tag: 7783-7084-3265-9085-8269-3286-77
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
\tOEM Information: 0x00000000
\tHeight: Unspecified
\tNumber Of Power Cords: Unspecified
\tContained Elements: 0
"""

DMIDECODE_FAIL = "# dmidecode 2.11\n# No SMBIOS nor DMI entry point found, sorry.\n"

DMIDECODE_ALIBABA = """
# dmidecode 3.2
Getting SMBIOS data from sysfs.
SMBIOS 2.8 present.
11 structures occupying 524 bytes.
Table at 0x000F4AC0.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
	Vendor: SeaBIOS
	Version: ab23bb1
	Release Date: 04/01/2014
	Address: 0xE8000
	Runtime Size: 96 kB
	ROM Size: 64 kB
	Characteristics:
		BIOS characteristics not supported
		Targeted content distribution is supported
	BIOS Revision: 0.0

Handle 0x0100, DMI type 1, 27 bytes
System Information
	Manufacturer: Alibaba Cloud
	Product Name: Alibaba Cloud ECS
	Version: pc-i440fx-2.1
	Serial Number: 1111111a-2220-333c-4449-555555555552
	UUID: 1111111a-2220-333c-4449-555555555552
	Wake-up Type: Power Switch
	SKU Number: Not Specified
	Family: Not Specified

Handle 0x0300, DMI type 3, 21 bytes
Chassis Information
	Manufacturer: Alibaba Cloud
	Type: Other
	Lock: Not Present
	Version: pc-i440fx-2.1
	Serial Number: Not Specified
	Asset Tag: Not Specified
	Boot-up State: Safe
	Power Supply State: Safe
	Thermal State: Safe
	Security Status: Unknown
	OEM Information: 0x00000000
	Height: Unspecified
	Number Of Power Cords: Unspecified
	Contained Elements: 0

Handle 0x0400, DMI type 4, 42 bytes
Processor Information
	Socket Designation: CPU 0
	Type: Central Processor
	Family: Other
	Manufacturer: Alibaba Cloud
	ID: 11 22 33 44 55 66 77 FF
	Version: pc-i440fx-2.1
	Voltage: Unknown
	External Clock: Unknown
	Max Speed: Unknown
	Current Speed: Unknown
	Status: Populated, Enabled
	Upgrade: Other
	L1 Cache Handle: Not Provided
	L2 Cache Handle: Not Provided
	L3 Cache Handle: Not Provided
	Serial Number: Not Specified
	Asset Tag: Not Specified
	Part Number: Not Specified
	Core Count: 2
	Core Enabled: 2
	Thread Count: 2
	Characteristics: None

Handle 0x1000, DMI type 16, 23 bytes
Physical Memory Array
	Location: Other
	Use: System Memory
	Error Correction Type: Multi-bit ECC
	Maximum Capacity: 32 GB
	Error Information Handle: Not Provided
	Number Of Devices: 2

Handle 0x1100, DMI type 17, 40 bytes
Memory Device
	Array Handle: 0x1000
	Error Information Handle: Not Provided
	Total Width: Unknown
	Data Width: Unknown
	Size: 16384 MB
	Form Factor: DIMM
	Set: None
	Locator: DIMM 0
	Bank Locator: Not Specified
	Type: RAM
	Type Detail: Other
	Speed: Unknown
	Manufacturer: Alibaba Cloud
	Serial Number: Not Specified
	Asset Tag: Not Specified
	Part Number: Not Specified
	Rank: Unknown
	Configured Memory Speed: Unknown
	Minimum Voltage: Unknown
	Maximum Voltage: Unknown
	Configured Voltage: Unknown

Handle 0x1101, DMI type 17, 40 bytes
Memory Device
	Array Handle: 0x1000
	Error Information Handle: Not Provided
	Total Width: Unknown
	Data Width: Unknown
	Size: 16384 MB
	Form Factor: DIMM
	Set: None
	Locator: DIMM 1
	Bank Locator: Not Specified
	Type: RAM
	Type Detail: Other
	Speed: Unknown
	Manufacturer: Alibaba Cloud
	Serial Number: Not Specified
	Asset Tag: Not Specified
	Part Number: Not Specified
	Rank: Unknown
	Configured Memory Speed: Unknown
	Minimum Voltage: Unknown
	Maximum Voltage: Unknown
	Configured Voltage: Unknown

Handle 0x1300, DMI type 19, 31 bytes
Memory Array Mapped Address
	Starting Address: 0x00000000000
	Ending Address: 0x000BFFFFFFF
	Range Size: 3 GB
	Physical Array Handle: 0x1000
	Partition Width: 1

Handle 0x1301, DMI type 19, 31 bytes
Memory Array Mapped Address
	Starting Address: 0x00100000000
	Ending Address: 0x0083FFFFFFF
	Range Size: 29 GB
	Physical Array Handle: 0x1000
	Partition Width: 1

Handle 0x2000, DMI type 32, 11 bytes
System Boot Information
	Status: No errors detected

Handle 0x7F00, DMI type 127, 4 bytes
End Of Table
"""  # noqa: E101,W191

IBM_RHSM_CONF = """
# Red Hat Subscription Manager Configuration File:

# Unified Entitlement Platform Configuration
[server]
# Server hostname:
hostname = host.networklayer.com

# Server prefix:
prefix = /subscription

# Server port:
port = 443

# Set to 1 to disable certificate validation:
insecure = 0

""".strip()


def test_rpm_google():
    irpms = IRPMS(context_wrap(RPMS_GOOGLE))
    dmi = DMIDecode(context_wrap(DMIDECODE))
    yrl = YumRepoList(context_wrap(YUM_REPOLIST_NOT_AZURE))
    ret = CloudProvider(irpms, dmi, yrl, None)
    assert ret.cloud_provider == CloudProvider.GOOGLE
    assert 'google-rhui-client-5.1.100-1.el7' in ret.cp_rpms.get(CloudProvider.GOOGLE)
    assert 'google-rhui-client-5.1.100-1.el6' in ret.cp_rpms.get(CloudProvider.GOOGLE)
    assert ret.long_name == 'Google Cloud'


def test_rpm_aws():
    irpms = IRPMS(context_wrap(RPMS_AWS))
    dmi = DMIDecode(context_wrap(DMIDECODE))
    yrl = YumRepoList(context_wrap(YUM_REPOLIST_NOT_AZURE))
    ret = CloudProvider(irpms, dmi, yrl, None)
    assert ret.cloud_provider == CloudProvider.AWS
    assert ret.cp_rpms.get(CloudProvider.AWS)[0] == 'rh-amazon-rhui-client-2.2.124-1.el7'
    assert ret.long_name == 'Amazon Web Services'


def test_rpm_azure():
    irpms = IRPMS(context_wrap(RPMS_AZURE))
    dmi = DMIDecode(context_wrap(DMIDECODE_BARE_METAL))
    yrl = YumRepoList(context_wrap(YUM_REPOLIST_NOT_AZURE))
    ret = CloudProvider(irpms, dmi, yrl, None)
    assert ret.cloud_provider == CloudProvider.AZURE
    assert ret.cp_rpms.get(CloudProvider.AZURE)[0] == 'WALinuxAgent-2.2.18-1.el7'
    assert ret.long_name == 'Microsoft Azure'


def test__yum_azure():
    irpms = IRPMS(context_wrap(RPMS))
    dmi = DMIDecode(context_wrap(DMIDECODE))
    yrl = YumRepoList(context_wrap(YUM_REPOLIST_AZURE))
    ret = CloudProvider(irpms, dmi, yrl, None)
    assert ret.cloud_provider == CloudProvider.AZURE
    assert 'rhui-microsoft-azure-rhel7-2.2-74' in ret.cp_yum.get(CloudProvider.AZURE)


def test__bios_version_aws():
    irpms = IRPMS(context_wrap(RPMS))
    dmi = DMIDecode(context_wrap(DMIDECODE_AWS))
    yrl = YumRepoList(context_wrap(YUM_REPOLIST_AZURE))
    ret = CloudProvider(irpms, dmi, yrl, None)
    assert ret.cloud_provider == CloudProvider.AWS
    assert ret.cp_bios_version[CloudProvider.AWS] == '4.2.amazon'


def test__bios_vendor_google():
    irpms = IRPMS(context_wrap(RPMS))
    dmi = DMIDecode(context_wrap(DMIDECODE_GOOGLE))
    yrl = YumRepoList(context_wrap(YUM_REPOLIST_AZURE))
    ret = CloudProvider(irpms, dmi, yrl, None)
    assert ret.cloud_provider == CloudProvider.GOOGLE
    assert ret.cp_bios_vendor[CloudProvider.GOOGLE] == 'Google'


def test__asset_tag_azure():
    irpms = IRPMS(context_wrap(RPMS))
    dmi = DMIDecode(context_wrap(DMIDECODE_AZURE_ASSET_TAG))
    yrl = YumRepoList(context_wrap(YUM_REPOLIST_NOT_AZURE))
    ret = CloudProvider(irpms, dmi, yrl, None)
    assert ret.cloud_provider == CloudProvider.AZURE
    assert ret.cp_asset_tag[CloudProvider.AZURE] == '7783-7084-3265-9085-8269-3286-77'


def test__uuid():
    irpms = IRPMS(context_wrap(RPMS))
    dmi = DMIDecode(context_wrap(DMIDECODE_AWS_UUID))
    yrl = YumRepoList(context_wrap(YUM_REPOLIST_NOT_AZURE))
    ret = CloudProvider(irpms, dmi, yrl, None)
    assert ret.cloud_provider == CloudProvider.AWS
    assert ret.cp_uuid[CloudProvider.AWS] == 'EC2F58AF-2DAD-C57E-88C0-A81CB6084290'


def test_dmidecode_alibaba():
    irpms = IRPMS(context_wrap(RPMS))
    dmi = DMIDecode(context_wrap(DMIDECODE_ALIBABA))
    yrl = YumRepoList(context_wrap(YUM_REPOLIST_NOT_AZURE))
    ret = CloudProvider(irpms, dmi, yrl, None)
    assert ret.cloud_provider == CloudProvider.ALIBABA
    assert ret.cp_manufacturer[CloudProvider.ALIBABA] == 'Alibaba Cloud'
    assert ret.long_name == 'Alibaba Cloud'


def test_rhsm_conf_ibm():
    rhsm_conf = RHSMConf(context_wrap(IBM_RHSM_CONF))
    ret = CloudProvider(None, None, None, rhsm_conf)
    assert ret.cloud_provider == CloudProvider.IBM
    assert ret.cp_rhsm_server_hostname[CloudProvider.IBM] == 'host.networklayer.com'
    assert ret.long_name == 'IBM Cloud'


def test_no_data():
    irpms = IRPMS(context_wrap(RPMS))
    dmi = DMIDecode(context_wrap(DMIDECODE))
    yrl = YumRepoList(context_wrap(YUM_REPOLIST_NOT_AZURE))
    ret = CloudProvider(irpms, dmi, yrl, None)
    assert ret.cloud_provider is None
    assert ret.long_name is None


def test_docs():
    cp_aws = CloudProvider(
        IRPMS(context_wrap(RPMS_AWS)),
        DMIDecode(context_wrap(DMIDECODE_AWS)),
        YumRepoList(context_wrap(YUM_REPOLIST_NOT_AZURE)),
        None
    )
    cp_azure = CloudProvider(
        IRPMS(context_wrap(RPMS_AZURE)),
        DMIDecode(context_wrap(DMIDECODE_AZURE_ASSET_TAG)),
        YumRepoList(context_wrap(YUM_REPOLIST_AZURE)),
        None
    )
    cp_alibaba = CloudProvider(
        IRPMS(context_wrap(RPMS)),
        DMIDecode(context_wrap(DMIDECODE_ALIBABA)),
        YumRepoList(context_wrap(YUM_REPOLIST_NOT_AZURE)),
        None
    )
    cp_ibm = CloudProvider(
        None, None, None, RHSMConf(context_wrap(IBM_RHSM_CONF))
    )
    env = {
        'cp_aws': cp_aws,
        'cp_azure': cp_azure,
        'cp_alibaba': cp_alibaba,
        'cp_ibm': cp_ibm
    }
    failed, total = doctest.testmod(cloud_provider, globs=env)
    assert failed == 0



import errno
import json
import os
import logging
import tempfile
import shlex
import shutil
import sys
import atexit
from subprocess import Popen, PIPE
from requests import ConnectionError

from .. import package_info
from . import client
from .constants import InsightsConstants as constants
from .config import InsightsConfig
from .auto_config import try_auto_configuration
from .utilities import (delete_registered_file,
                        delete_unregistered_file,
                        write_to_disk,
                        generate_machine_id,
                        get_tags,
                        write_tags,
                        migrate_tags,
                        get_parent_process)

NETWORK = constants.custom_network_log_level
logger = logging.getLogger(__name__)


class InsightsClient(object):

    def __init__(self, config=None, from_phase=True, **kwargs):
        """
        The Insights client interface
        """
        if config is None:
            # initialize with default config if not specified with one
            self.config = InsightsConfig()
        else:
            # BEGIN small hack to maintain compatibility with RPM wrapper:
            #   wrapper calls with bool second arg so be smart about it
            if isinstance(config, InsightsConfig):
                self.config = config
            else:
                try:
                    self.config = InsightsConfig(_print_errors=True).load_all()
                except ValueError as e:
                    sys.stderr.write('ERROR: ' + str(e) + '\n')
                    sys.exit(constants.sig_kill_bad)
            # END hack. in the future, just set self.config=config

        if from_phase:
            _init_client_config_dirs()
            self.set_up_logging()
            try_auto_configuration(self.config)
            self.initialize_tags()
        else:  # from wrapper
            _write_pid_files()

        # setup insights connection placeholder
        # used for requests
        self.session = None
        self.connection = None
        self.tmpdir = None

    def _net(func):
        def _init_connection(self, *args, **kwargs):
            # setup a request session
            if not self.config.offline and not self.session:
                self.connection = client.get_connection(self.config)
                self.session = self.connection.session
            return func(self, *args, **kwargs)
        return _init_connection

    def get_conf(self):
        # need this getter to maintain compatability with RPM wrapper
        return self.config

    def set_up_logging(self):
        return client.set_up_logging(self.config)

    def version(self):
        return "%s-%s" % (package_info["VERSION"], package_info["RELEASE"])

    @_net
    def test_connection(self):
        """
            returns (int): 0 if success 1 if failure
        """
        return self.connection.test_connection()

    @_net
    def branch_info(self):
        """
            returns (dict): {'remote_leaf': -1, 'remote_branch': -1}
        """
        return client.get_branch_info(self.config, self.connection)

    @_net
    def get_egg_url(self):
        """
        Get the proper url based on the configured egg release branch
        """
        if self.config.legacy_upload:
            url = self.connection.base_url + '/platform' + constants.module_router_path
        else:
            url = self.connection.base_url + constants.module_router_path
        logger.log(NETWORK, "GET %s", url)
        response = self.session.get(url, timeout=self.config.http_timeout)
        if response.status_code == 200:
            return response.json()["url"]
        else:
            logger.warning("Unable to fetch egg url. Defaulting to /release")
            return '/release'

    def fetch(self, force=False):
        """
            returns (dict): {'core': path to new egg, None if no update,
                             'gpg_sig': path to new sig, None if no update}
        """
        self.tmpdir = tempfile.mkdtemp()
        atexit.register(self.delete_tmpdir)
        fetch_results = {
            'core': os.path.join(self.tmpdir, 'insights-core.egg'),
            'gpg_sig': os.path.join(self.tmpdir, 'insights-core.egg.asc')
        }

        logger.debug("Beginning core fetch.")

        # guess the URLs based on what legacy setting is
        egg_release = self.get_egg_url()

        try:
            # write the release path to temp so we can collect it
            #   in the archive
            write_to_disk(constants.egg_release_file, content=egg_release)
        except (OSError, IOError) as e:
            logger.debug('Could not write egg release file: %s', str(e))

        egg_url = self.config.egg_path
        egg_gpg_url = self.config.egg_gpg_path
        if egg_url is None:
            egg_url = '/v1/static{0}/insights-core.egg'.format(egg_release)
            # if self.config.legacy_upload:
            #     egg_url = '/v1/static/core/insights-core.egg'
            # else:
            #     egg_url = '/static/insights-core.egg'
        if egg_gpg_url is None:
            egg_gpg_url = '/v1/static{0}/insights-core.egg.asc'.format(egg_release)
            # if self.config.legacy_upload:
            #     egg_gpg_url = '/v1/static/core/insights-core.egg.asc'
            # else:
            #     egg_gpg_url = '/static/insights-core.egg.asc'
        # run fetch for egg
        updated = self._fetch(egg_url,
                              constants.core_etag_file,
                              fetch_results['core'],
                              force)

        # if new core was fetched, get new core sig
        if updated:
            logger.debug("New core was fetched.")
            logger.debug("Beginning fetch for core gpg signature.")
            self._fetch(egg_gpg_url,
                        constants.core_gpg_sig_etag_file,
                        fetch_results['gpg_sig'],
                        force)

            return fetch_results

    @_net
    def _fetch(self, path, etag_file, target_path, force):
        """
            returns (str): path to new egg. None if no update.
        """
        # Searched for cached etag information
        current_etag = None
        if os.path.isfile(etag_file):
            with open(etag_file, 'r') as fp:
                current_etag = fp.read().strip()
                logger.debug('Found etag %s', current_etag)

        # it's only temporary. I promise. this is the worst timeline
        # all for a phone popup
        if self.config.legacy_upload:
            url = self.connection.base_url + path
            # verify = self.config.cert_verify
        else:
            url = self.connection.base_url.split('/platform')[0] + path
            # if self.config.cert_verify is True:
            #     # dont overwrite satellite cert
            #     self.cert_verify = os.path.join(
            #         constants.default_conf_dir,
            #         'cert-api.access.redhat.com.pem')

        # Setup the new request for core retrieval
        logger.debug('Making request to %s for new core', url)

        # If the etag was found and we are not force fetching
        # Then add it to the request
        logger.log(NETWORK, "GET %s", url)
        try:
            if current_etag and not force:
                logger.debug('Requesting new file with etag %s', current_etag)
                etag_headers = {'If-None-Match': current_etag}
                response = self.session.get(url, headers=etag_headers, timeout=self.config.http_timeout)
            else:
                logger.debug('Found no etag or forcing fetch')
                response = self.session.get(url, timeout=self.config.http_timeout)
        except ConnectionError as e:
            logger.error(e)
            logger.error('The Insights API could not be reached.')
            return False

        # Debug information
        logger.debug('Status code: %d', response.status_code)
        for header, value in response.headers.items():
            logger.debug('%s: %s', header, value)

        # Debug the ETag
        logger.debug('ETag: %s', response.request.headers.get('If-None-Match'))

        # If data was received, write the new egg and etag
        if response.status_code == 200 and len(response.content) > 0:

            # Write the new core
            with open(target_path, 'wb') as handle:
                logger.debug('Data received, writing core to %s', target_path)
                handle.write(response.content)

            # Write the new etag
            with open(etag_file, 'w') as handle:
                logger.debug('Cacheing etag to %s', etag_file)
                handle.write(response.headers['etag'])

            return True

        # Received a 304 not modified
        # Return nothing
        elif response.status_code == 304:
            logger.debug('No data received')
            logger.debug('Tags match, not updating core')

        # Something unexpected received
        else:
            logger.debug('Received Code %s', response.status_code)
            logger.debug('Not writing new core, or updating etag')
            logger.debug('Please check config, error reaching %s', url)

    def update(self):
        # dont update if running in offline mode
        if self.config.offline:
            logger.debug("Not updating Core. Running in offline mode.")
            return True

        if self.config.auto_update:
            # fetch the new eggs and gpg
            egg_paths = self.fetch()

            # if the gpg checks out install it
            if (egg_paths and self.verify(egg_paths['core'])['gpg']):
                return self.install(egg_paths['core'], egg_paths['gpg_sig'])
            else:
                return False
        else:
            logger.debug("Egg update disabled")

    def verify(self, egg_path, gpg_key=constants.pub_gpg_path):
        """
            Verifies the GPG signature of the egg.  The signature is assumed to
            be in the same directory as the egg and named the same as the egg
            except with an additional ".asc" extension.

            returns (dict): {'gpg': if the egg checks out,
                             'stderr': error message if present,
                             'stdout': stdout,
                             'rc': return code}
        """
        # check if the provided files (egg and gpg) actually exist
        if egg_path and not os.path.isfile(egg_path):
            the_message = "Provided egg path %s does not exist, cannot verify." % (egg_path)
            logger.debug(the_message)
            return {'gpg': False,
                    'stderr': the_message,
                    'stdout': the_message,
                    'rc': 1,
                    'message': the_message}
        if self.config.gpg and gpg_key and not os.path.isfile(gpg_key):
            the_message = ("Running in GPG mode but cannot find "
                            "file %s to verify against." % (gpg_key))
            logger.debug(the_message)
            return {'gpg': False,
                    'stderr': the_message,
                    'stdout': the_message,
                    'rc': 1,
                    'message': the_message}

        # if we are running in no_gpg or not gpg mode then return true
        if not self.config.gpg:
            return {'gpg': True,
                    'stderr': None,
                    'stdout': None,
                    'rc': 0}

        # if a valid egg path and gpg were received do the verification
        if egg_path and gpg_key:
            cmd_template = '/usr/bin/gpg --verify --keyring %s %s %s'
            cmd = cmd_template % (gpg_key, egg_path + '.asc', egg_path)
            logger.debug(cmd)
            process = Popen(shlex.split(cmd), stdout=PIPE, stderr=PIPE)
            stdout, stderr = process.communicate()
            rc = process.returncode
            logger.debug("GPG return code: %s" % rc)
            return {'gpg': True if rc == 0 else False,
                    'stderr': stderr,
                    'stdout': stdout,
                    'rc': rc}
        else:
            return {'gpg': False,
                    'stderr': 'Must specify a valid core and gpg key.',
                    'stdout': 'Must specify a valid core and gpg key.',
                    'rc': 1}

    def install(self, new_egg, new_egg_gpg_sig):
        """
        returns (dict): {'success': True if the core installation successfull else False}
        raises OSError if cannot create /var/lib/insights
        raises IOError if cannot copy /tmp/insights-core.egg to /var/lib/insights/newest.egg
        """

        # make sure a valid egg was provided
        if not new_egg:
            the_message = 'Must provide a valid Core installation path.'
            logger.debug(the_message)
            return {'success': False, 'message': the_message}

        # if running in gpg mode, check for valid sig
        if self.config.gpg and new_egg_gpg_sig is None:
            the_message = "Must provide a valid Core GPG installation path."
            logger.debug(the_message)
            return {'success': False, 'message': the_message}

        # debug
        logger.debug("Installing the new Core %s", new_egg)
        if self.config.gpg:
            logger.debug("Installing the new Core GPG Sig %s", new_egg_gpg_sig)

        # Make sure /var/lib/insights exists
        try:
            if not os.path.isdir(constants.insights_core_lib_dir):
                logger.debug("Creating directory %s for the Core." %
                             (constants.insights_core_lib_dir))
                os.mkdir(constants.insights_core_lib_dir)
        except OSError:
            logger.info("There was an error creating %s for core installation." % (
                constants.insights_core_lib_dir))
            raise

        # Copy the NEW (/tmp/insights-core.egg) egg to /var/lib/insights/newest.egg
        # Additionally, copy NEW (/tmp/insights-core.egg.asc) to /var/lib/insights/newest.egg.asc
        try:
            logger.debug("Copying %s to %s." % (new_egg, constants.insights_core_newest))
            shutil.copyfile(new_egg, constants.insights_core_newest)
            shutil.copyfile(new_egg_gpg_sig, constants.insights_core_gpg_sig_newest)
        except IOError:
            logger.info("There was an error copying the new core from %s to %s." % (
                new_egg, constants.insights_core_newest))
            raise

        logger.debug("The new Insights Core was installed successfully.")
        return {'success': True}

    def delete_tmpdir(self):
        if self.tmpdir:
            logger.debug("Deleting temp directory %s." % (self.tmpdir))
            shutil.rmtree(self.tmpdir, True)

    @_net
    def update_rules(self):
        """
            returns (dict): new client rules
        """
        if self.config.offline or not self.config.auto_update:
            logger.debug("Bypassing rule update due to config "
                "running in offline mode or auto updating turned off.")
        else:
            return client.update_rules(self.config, self.connection)

    @_net
    def collect(self):
        # return collection results
        tar_file = client.collect(self.config, self.connection)

        # it is important to note that --to-stdout is utilized via the wrapper RPM
        # this file is received and then we invoke shutil.copyfileobj
        return tar_file

    @_net
    def register(self):
        """
            returns (bool | None):
                True - machine is registered
                False - machine is unregistered
                None - could not reach the API
        """
        return client.handle_registration(self.config, self.connection)

    @_net
    def unregister(self):
        """
            returns (bool): True success, False failure
        """
        return client.handle_unregistration(self.config, self.connection)

    @_net
    def upload(self, payload=None, content_type=None):
        """
            Upload the archive at `path` with content type `content_type`
            returns (int): upload status code
        """
        # platform - prefer the value passed in to func over config
        payload = payload or self.config.payload
        content_type = content_type or self.config.content_type

        if payload is None:
            raise ValueError('Specify a file to upload.')

        if not os.path.exists(payload):
            raise IOError('Cannot upload %s: File does not exist.' % payload)

        upload_results = client.upload(
            self.config, self.connection, payload, content_type)

        # return api response
        return upload_results

    def rotate_eggs(self):
        """
            moves newest.egg to last_stable.egg
            this is used by the upload() function upon 2XX return
            returns (bool): if eggs rotated successfully
            raises (IOError): if it cant copy the egg from newest to last_stable
        """
        # make sure the library directory exists
        if os.path.isdir(constants.insights_core_lib_dir):
            # make sure the newest.egg exists
            if os.path.isfile(constants.insights_core_newest):
                # try copying newest to latest_stable
                try:
                    # copy the core
                    shutil.move(constants.insights_core_newest,
                             constants.insights_core_last_stable)
                    # copy the core sig
                    shutil.move(constants.insights_core_gpg_sig_newest,
                             constants.insights_core_last_stable_gpg_sig)
                except IOError:
                    message = ("There was a problem copying %s to %s." %
                                (constants.insights_core_newest,
                                constants.insights_core_last_stable))
                    logger.debug(message)
                    raise IOError(message)
                return True
            else:
                message = ("Cannot copy %s to %s because %s does not exist." %
                            (constants.insights_core_newest,
                            constants.insights_core_last_stable,
                            constants.insights_core_newest))
                logger.debug(message)
                return False
        else:
            logger.debug("Cannot copy %s to %s because the %s directory does not exist." %
                (constants.insights_core_newest,
                    constants.insights_core_last_stable,
                    constants.insights_core_lib_dir))
            logger.debug("Try installing the Core first.")
            return False

    def get_last_upload_results(self):
        """
            returns (json): returns last upload json results or False
        """
        if os.path.isfile(constants.last_upload_results_file):
            logger.debug('Last upload file %s found, reading results.', constants.last_upload_results_file)
            with open(constants.last_upload_results_file, 'r') as handler:
                return handler.read()
        else:
            logger.debug('Last upload file %s not found, cannot read results', constants.last_upload_results_file)
            return False

    @_net
    def get_registration_status(self):
        """
            returns (json):
                {'messages': [dotfile message, api message],
                 'status': (bool) registered = true; unregistered = false
                 'unreg_date': Date the machine was unregistered | None,
                 'unreachable': API could not be reached}
        """
        return client.get_registration_status(self.config, self.connection)

    @_net
    def set_display_name(self, display_name):
        '''
            returns True on success, False on failure
        '''
        return self.connection.set_display_name(display_name)

    @_net
    def get_diagnosis(self, remediation_id=None):
        '''
            returns JSON of diagnosis data on success, None on failure
            Optional arg remediation_id to get a particular remediation set.
        '''
        if self.config.offline:
            logger.error('Cannot get diagnosis in offline mode.')
            return None
        return self.connection.get_diagnosis(remediation_id)

    def delete_cached_branch_info(self):
        '''
            Deletes cached branch_info file
        '''
        if os.path.isfile(constants.cached_branch_info):
            logger.debug('Deleting cached branch_info file...')
            os.remove(constants.cached_branch_info)
        else:
            logger.debug('Cached branch_info file does not exist.')

    def get_machine_id(self):
        return client.get_machine_id()

    def clear_local_registration(self):
        '''
        Deletes dotfiles and machine-id for fresh registration
        '''
        delete_registered_file()
        delete_unregistered_file()
        write_to_disk(constants.machine_id_file, delete=True)
        logger.debug('Re-register set, forcing registration.')
        logger.debug('New machine-id: %s', generate_machine_id(new=True))

    @_net
    def check_results(self):
        content = self.connection.get_advisor_report()
        if content is None:
            raise Exception("Error: failed to download advisor report.")

    def show_results(self):
        '''
        Show insights about this machine
        '''
        try:
            with open("/var/lib/insights/insights-details.json", mode="r+b") as f:
                insights_data = json.load(f)
            print(json.dumps(insights_data, indent=1))
        except IOError as e:
            if e.errno == errno.ENOENT:
                raise Exception("Error: no report found. Run insights-client --check-results to update the report cache: %s" % e)
            else:
                raise e

    def show_inventory_deep_link(self):
        """
        Show a deep link to this host inventory record
        """
        system = self.connection._fetch_system_by_machine_id()
        if system:
            if len(system) == 1:
                try:
                    id = system[0]["id"]
                    logger.info("View details about this system on cloud.redhat.com:")
                    logger.info(
                        "https://cloud.redhat.com/insights/inventory/{0}".format(id)
                    )
                except Exception as e:
                    logger.error(
                        "Error: malformed system record: {0}: {1}".format(system, e)
                    )

    def _copy_soscleaner_files(self, insights_archive):
        '''
        Helper function to copy the .csv reports generated by SOScleaner

        Parameters:
            insights_archive - source path of the scrubbed data in temp
        '''
        src_dir = os.path.dirname(insights_archive)
        dst_file_prefix = self.config.output_dir if self.config.output_dir else self.config.output_file

        for fil in os.listdir(src_dir):
            if fil.endswith('.csv'):
                file_suffix = fil.rsplit('-', 1)[1]
                src_path = os.path.join(src_dir, fil)
                dst_path = dst_file_prefix + '-' + file_suffix
                try:
                    if os.path.exists(dst_path):
                        # don't overwrite anything arbitrary
                        raise OSError('%s already exists.' % dst_path)
                    logger.debug('Copying SOScleaner report from %s to %s', src_path, dst_path)
                    shutil.copyfile(src_path, dst_path)
                    logger.info('SOScleaner report copied to %s', dst_path)
                except OSError as e:
                    logger.error('ERROR: Could not write data to %s', dst_path)
                    logger.error(e)

    def copy_to_output_dir(self, insights_archive):
        '''
        Copy the collected data from temp to the directory
        specified by --output-dir

        Parameters:
            insights_archive - the path to the collected dir
        '''
        logger.debug('Copying collected data from %s to %s',
                     insights_archive, self.config.output_dir)
        try:
            shutil.copytree(insights_archive, self.config.output_dir)
        except OSError as e:
            if e.errno == 17:
                # dir exists already, see if it's empty
                if os.listdir(self.config.output_dir):
                    # we should never get here because of the check in config.py, but just in case
                    logger.error('ERROR: Could not write data to %s.', self.config.output_dir)
                    logger.error(e)
                    return
                else:
                    # if it's empty, copy the contents to it
                    for fil in os.listdir(insights_archive):
                        src_path = os.path.join(insights_archive, fil)
                        dst_path = os.path.join(self.config.output_dir, fil)
                        try:
                            if os.path.isfile(src_path):
                                shutil.copyfile(src_path, dst_path)
                            elif os.path.isdir(src_path):
                                shutil.copytree(src_path, dst_path)
                        except OSError as e:
                            logger.error(e)
                            # in case this happens partway through let the user know
                            logger.warning('WARNING: Directory copy may be incomplete.')
                            return
            else:
                # some other error
                logger.error(e)
                return
        logger.info('Collected data copied to %s', self.config.output_dir)
        if self.config.obfuscate:
            self._copy_soscleaner_files(insights_archive)

    def copy_to_output_file(self, insights_archive):
        '''
        Copy the collected archive from temp to the file
        specified by output-file

        insights_archive - the path to the collected archive
        '''
        logger.debug('Copying archive from %s to %s',
                     insights_archive, self.config.output_file)
        try:
            shutil.copyfile(insights_archive, self.config.output_file)
        except OSError as e:
            # file exists already
            logger.error('ERROR: Could not write data to %s', self.config.output_file)
            logger.error(e)
            return
        logger.info('Collected data copied to %s', self.config.output_file)
        if self.config.obfuscate:
            self._copy_soscleaner_files(insights_archive)

    def initialize_tags(self):
        '''
        Initialize the tags file if needed
        '''
        # migrate the old file if necessary
        migrate_tags()

        # initialize with group if group was specified
        if self.config.group:
            tags = get_tags()
            if tags is None:
                tags = {}
            tags["group"] = self.config.group
            write_tags(tags)

    def list_specs(self):
        logger.info("For a full list of insights-core datasources, please refer to https://insights-core.readthedocs.io/en/latest/specs_catalog.html")
        logger.info("The items in General Datasources can be selected for omission by adding them to the 'components' section of file-redaction.yaml")
        logger.info("When specifying these items in file-redaction.yaml, they must be prefixed with 'insights.specs.default.DefaultSpecs.', i.e. 'insights.specs.default.DefaultSpecs.httpd_V'")
        logger.info("This information applies only to Insights Core collection. To use Core collection, set core_collect=True in %s", self.config.conf)

    @_net
    def checkin(self):
        if self.config.offline:
            logger.error('Cannot check-in in offline mode.')
            return None

        return self.connection.checkin()


def format_config(config):
    # Log config except the password
    # and proxy as it might have a pw as well
    config_copy = config.copy()
    try:
        del config_copy["password"]
        del config_copy["proxy"]
    finally:
        return json.dumps(config_copy, indent=4)


def _init_client_config_dirs():
    '''
    Initialize log and lib dirs
    TODO: init non-root config dirs
    '''
    for d in (constants.log_dir, constants.insights_core_lib_dir):
        try:
            os.makedirs(d)
        except OSError as e:
            if e.errno == errno.EEXIST:
                # dir exists, this is OK
                pass
            else:
                raise e


def _write_pid_files():
    for file, content in (
        (constants.pidfile, str(os.getpid())),  # PID in case we need to ping systemd
        (constants.ppidfile, get_parent_process())  # PPID so that we can grab the client execution method
    ):
        write_to_disk(file, content=content)
        atexit.register(write_to_disk, file, delete=True)



"""
NeutronSriovAgent - file ``/etc/neutron/plugins/ml2/sriov_agent.ini``
=====================================================================

This class provides parsing for the files:
    ``/etc/neutron/plugins/ml2/sriov_agent.ini``
    ``/var/lib/config-data/puppet-generated/neutron/etc/neutron/plugins/ml2/sriov_agent.ini``

Sample input data is in the format::

    [DEFAULT]
    debug = false

    [sriov_nic]
    physical_device_mappings=datacentre:enp2s0f6

    [agent]
    polling_interval=2

    [securitygroup]
    firewall_driver=noop
    report_interval = 60

    [keystone_authtoken]
    auth_port = 35357

See the ``IniConfigFile`` class for examples.
"""
from .. import IniConfigFile, parser, add_filter
from insights.specs import Specs

FILTERS = [
    "debug",
    "[",
    "physical_device_mappings",
    "exclude_devices",
    "extensions"
]
add_filter(Specs.neutron_sriov_agent, FILTERS)


@parser(Specs.neutron_sriov_agent)
class NeutronSriovAgent(IniConfigFile):
    """Class to parse file ``sriov_agent.ini``."""
    pass



import sys
from threading import Thread
import socket
import random

if (sys.version_info > (3, 0)):
    from http.server import BaseHTTPRequestHandler, HTTPServer
else:
    from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler


def get_free_port():
    """
    Retrieves a free port for the mock service to use

    :return (int): free port
    """
    s = socket.socket(socket.AF_INET, type=socket.SOCK_STREAM)
    s.bind(('localhost', 0))
    address, port = s.getsockname()
    s.close()
    return port


SERVER_PORT = get_free_port()


class RequestHandler(BaseHTTPRequestHandler):
    """
    BaseHTTPRequestHandler subclass configured to handle GET requests for the TestMockServer.

    """
    def do_GET(self):
        """
        Handles the get call from the HTTPServer

        """

        print(self.path)
        value = ''
        send_reply = False
        if self.path.endswith("/mock/"):
            send_reply = True

        value = "text/json"

        if send_reply:
            self.send_response(200)
            self.send_header('Content-type', value)
            self.end_headers()
            num = random.randint(1, 100000)
            output = ""
            output += '{"data":{"id": ' + str(num) + ', "name": "Successful return from Mock Service"}}'
            self.wfile.write(output.encode('utf_8'))
            print(output)
            return output
        else:
            self.send_response(404)
            self.send_header('Content-type', value)
            self.end_headers()
            output = ""
            output += '{"error":{"code": "404", "message": "Not Found"}}'
            self.wfile.write(output.encode('utf_8'))
            print(output)
            return
        return


class TestMockServer(object):
    """
    TestMockServer subclass for accessing external Web resources using caching.

    Examples:

        >>> from insights.tests.mock_web_server import TestMockServer, RequestHandler

        >>> crr = CachedRemoteResource()
        >>> sv = TestMockServer()
        >>> sv.setup_class()
        >>> sv.get_server_port()
        35917
        >>> curl -X GET http://localhost:35917/mock/
        {"data":{"id": "001", "name": "Successful return from Mock Service"}}
    """

    @classmethod
    def setup_class(cls):
        """
        Class mathod that creates the Test Service daemon

        """
        cls.server_port = get_free_port()
        cls.mock_server_port = get_free_port()
        cls.mock_server = HTTPServer(('localhost', cls.server_port), RequestHandler)

        # Start running mock server in a separate thread.
        # Daemon threads automatically shut down when the main process exits.
        cls.mock_server_thread = Thread(target=cls.mock_server.serve_forever)
        cls.mock_server_thread.setDaemon(True)
        cls.mock_server_thread.start()

    def get_server_port(self):
        """
        Helper function to get the port that was used to start the service.

        """

        return self.server_port



"""
Mdstat - file ``/proc/mdstat``
==============================

Represents the information in the ``/proc/mdstat`` file.  Several
examples of possible data containe in the file can be found on the
`MDstat kernel.org wiki page <https://raid.wiki.kernel.org/index.php/Mdstat>`_.

In particular, the discussion here will focus on initial extraction of information
form lines such as::

    Personalities : [raid1] [raid6] [raid5] [raid4]
    md1 : active raid1 sdb2[1] sda2[0]
          136448 blocks [2/2] [UU]

    md2 : active raid1 sdb3[1] sda3[0]
          129596288 blocks [2/2] [UU]

    md3 : active raid5 sdl1[9] sdk1[8] sdj1[7] sdi1[6] sdh1[5] sdg1[4] sdf1[3] sde1[2] sdd1[1] sdc1[0]
          1318680576 blocks level 5, 1024k chunk, algorithm 2 [10/10] [UUUUUUUUUU]

The data contained in ``mdstat`` is represented with three top level members -
``personalities``, ``components`` and ``mds``.

Examples:

    >>> mdstat = shared[Mdstat]
    >>> mdstat.personalities
    ['raid1', 'raid6', 'raid5', 'raid4']
    >>> len(mdstat.components) # The individual component devices
    14
    >>> mdstat.components[0]['device_name']
    'md1'
    >>> sdb2 = mdstat.components[0]
    >>> sdb2['component_name']
    'sdb2'
    >>> sdb2['active']
    True
    >>> sdb2['raid']
    'raid1'
    >>> sdb2['role']
    1
    >>> sdb2['up']
    True
    >>> sorted(mdstat.mds.keys()) # dictionary of MD devices by device name
    ['md1', 'md2', 'md3']
    >>> mdstat.mds['md1']['active']
    True
    >>> len(mdstat.mds['md1']['devices']) # list of devices in this MD
    2
    >>> mdstat.mds['md1']['devices'][0]['component_name'] # device information
    'sdb2'
"""
import re
from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.mdstat)
class Mdstat(CommandParser):
    """
    Represents the information in the ``/proc/mdstat`` file.

    Attributes
    ----------

    personalities : list
        A list of RAID levels the kernel currently supports

    components : list of dicts
        A list containing a dict of md component device information
        Each of these dicts contains the following keys

        - ``device_name`` : string - name of the array device
        - ``active`` : boolean - ``True`` if the array is active, ``False``
          if it is inactive.
        - ``component_name`` : string - name of the component device
        - ``raid`` : string - with the raid level, e.g., "raid1" for "md1"
        - ``role`` : int - raid role number
        - ``device_flag`` : str - device component status flag.  Known values
          include 'F' (failed device), 'S', and 'W'
        - ``up`` : boolean - ``True`` if the component device is up
        - ``auto_read_only`` : boolean - ``True`` if the array device is
          "auto-read-only"
        - ``blocks`` : the number of blocks in the device
        - ``level`` : the current RAID level, if found in the status line
        - ``chunk`` : the device chunk size, if found in the status line
        - ``algorithm`` : the current conflict resolution algorithm, if found
          in the status line

    mds : dict of dicts
        A dictionary keyed on the MD device name, with the following keys

         - ``name``: Name of the MD device
         - ``active``: Whether the MD device is active
         - ``raid``: The RAID type string
         - ``devices``: a list of the devices in this
         - ``blocks``, ``level``, ``chunk`` and ``algorithm`` - the same
           information given above per component device (if found)

    """

    def parse_content(self, content):
        self.mds = {}
        self.components = []
        self.personalities = []

        current_components = None
        in_component = False

        for line in content:
            line = line.strip()
            if line.startswith('Personalities'):
                in_component = False
                self.personalities = parse_personalities(line)
            elif line.startswith("md"):  # Starting a component array stanza
                in_component = True
                current_components = parse_array_start(line)
            elif not line:  # blank line, ending a component array stanza
                if in_component:
                    self.components.extend(current_components)
                    current_components = None
                in_component = False
            else:
                if in_component:
                    parse_array_status(line, current_components)
                    upstring = parse_upstring(line)
                    if upstring:
                        apply_upstring(upstring, current_components)

        # Map component devices into MDs dictionary by device name
        for comp in self.components:
            devname = comp['device_name']
            if devname not in self.mds:
                #     md2 : active raid1 sdb3[1] sda3[0]
                #           129596288 blocks [2/2] [UU]
                self.mds[devname] = {
                    'name': devname, 'active': comp['active'],
                    'raid': comp['raid'], 'blocks': comp['blocks'],
                    'devices': []
                }
                for opt in ['level', 'chunk', 'algorithm']:
                    if opt in comp:
                        self.mds[devname][opt] = comp[opt]
            self.mds[devname]['devices'].append(dict(
                (k, comp[k]) for k in comp if k in ['component_name', 'role', 'up']
            ))

        # Keep self.data just for backwards compat
        self.data = {
            'personalities': self.personalities,
            'components': self.components
        }


def parse_personalities(personalities_line):
    """Parse the "personalities" line of ``/proc/mdstat``.

    Lines are expected to be like:

        Personalities : [linear] [raid0] [raid1] [raid5] [raid4] [raid6]

    If they do not have this format, an error will be raised since it
    would be considered an unexpected parsing error.

    Parameters
    ----------

    personalities_line : str
        A single "Personalities" line from an ``/proc/mdstat`` files.

    Returns
    -------
        A list of raid "personalities" listed on the line.
    """
    tokens = personalities_line.split()
    assert tokens.pop(0) == "Personalities"
    assert tokens.pop(0) == ":"

    personalities = []
    for token in tokens:
        assert token.startswith('[') and token.endswith(']')
        personalities.append(token.strip('[]'))

    return personalities


def parse_array_start(md_line):
    """Parse the initial line of a device array stanza in
    ``/proc/mdstat``.

    Lines are expected to be like:

        md2 : active raid1 sdb3[1] sda3[0]

    If they do not have this format, an error will be raised since it
    would be considered an unexpected parsing error.

    Parameters
    ----------

    md_line : str
        A single line from the start of a device array stanza from a
        ``/proc/mdstat`` file.

    Returns
    -------
        A list of dictionaries, one dictionrary for each component
        device making up the array.
    """
    components = []
    tokens = md_line.split()
    device_name = tokens.pop(0)
    assert device_name.startswith("md")
    assert tokens.pop(0) == ":"

    active_string = tokens.pop(0)
    active = False
    if active_string == "active":
        active = True
    else:
        assert active_string == "inactive"

    raid_read_only_token = tokens.pop(0)
    auto_read_only = False
    raid = None
    if raid_read_only_token == "(auto-read-only)":
        auto_read_only = True
        raid = tokens.pop(0)
    elif active:
        raid = raid_read_only_token
    else:  # Inactive devices don't list the raid type
        raid = None
        tokens.insert(0, raid_read_only_token)

    for token in tokens:
        subtokens = re.split(r"\[|]", token)
        assert len(subtokens) > 1
        comp_name = subtokens[0]
        assert comp_name

        role = int(subtokens[1])

        device_flag = None
        if len(subtokens) > 2:
            device_flag = subtokens[2]
            if device_flag:
                device_flag = device_flag.strip('()')

        component_row = {"device_name": device_name,
                         "raid": raid,
                         "active": active,
                         "auto_read_only": auto_read_only,
                         "component_name": comp_name,
                         "role": role,
                         "device_flag": device_flag}
        components.append(component_row)
    return components


def parse_array_status(line, components):
    """
    Parse the array status line, e.g.::

        1318680576 blocks level 5, 1024k chunk, algorithm 2 [10/10] [UUUUUUUUUU]

    This retrieves the following pieces of information:

    * ``blocks`` - (int) number of blocks in the whole MD device (always present)
    * ``level`` - (int) if found, the present RAID level
    * ``chunksize`` - (str) if found, the size of the data chunk in kilobytes
    * ``algorithm`` - (int) if found, the current algorithm in use.

    Because of the way data is stored per-component and not per-array, this
    then puts the above keys into each of the component dictionaries in the
    list we've been given.

    Sample data::

        1250241792 blocks super 1.2 level 5, 64k chunk, algorithm 2 [5/5] [UUUUUU]
        1465151808 blocks level 5, 64k chunk, algorithm 2 [4/3] [UUU_]
        136448 blocks [2/2] [UU]
        6306 blocks super external:imsm<Paste>
    """
    status_line_re = r'(?P<blocks>\d+) blocks' + \
        r'(?: super (?P<super>\S+))?' + \
        r'(?: level (?P<level>\d+),)?' + \
        r'(?: (?P<chunk>\d+k) chunk,)?' + \
        r'(?: algorithm (?P<algorithm>\d+))?'
    # Since we're only called once per line, unless there's a good way to
    # cache this regular expression in compiled form we're going to have to
    # compile it each time.
    status_line_rex = re.compile(status_line_re)
    match = status_line_rex.search(line)
    if match:
        attributes = {'blocks': int(match.group('blocks'))}
        if match.group('level'):
            attributes['level'] = int(match.group('level'))
        if match.group('chunk'):
            attributes['chunk'] = match.group('chunk')
        if match.group('algorithm'):
            attributes['algorithm'] = int(match.group('algorithm'))
        for comp in components:
            comp.update(attributes)


def parse_upstring(line):
    """
    Parse the subsequent lines of a device array stanza in ``/proc/mdstat``
    for the "up" indictor string.

    Lines are expected to be like:

        129596288 blocks [2/2] [UU]

    or

        1318680576 blocks level 5, 1024k chunk, algorithm 2 [10/10] [UUU_UUUUUU]

    In particular, this method searchs for the string like ``[UU]`` which
    indicates whether component devices or up, ``U`` or down, ``_``.

    Parameters
    ----------

    line : str
        A single line from a device array stanza.

    Returns
    -------

        The string containing a series of ``U`` and ``\_`` characters if
        found in the string, and ``None`` if the uptime string is not found.
    """
    UP_STRING_REGEX = r"\[(U|_)+]"

    match = re.search(UP_STRING_REGEX, line)
    if match:
        return match.group().strip('[]')
    else:
        return None


def apply_upstring(upstring, component_list):
    """Update the dictionaries resulting from ``parse_array_start`` with
    the "up" key based on the upstring returned from ``parse_upstring``.

    The function assumes that the upstring and component_list parameters
    passed in are from the same device array stanza of a
    ``/proc/mdstat`` file.

    The function modifies component_list in place, adding or updating
    the value of the "up" key to True if there is a corresponding ``U``
    in the upstring string, or to False if there is a corresponding
    ``_``.

    If there the number of rows in component_list does not match the
    number of characters in upstring, an ``AssertionError`` is raised.

    Parameters
    ----------

    upstring : str
        String sequence of ``U``s and ``_``s as determined by the
        ``parse_upstring`` method

    component_list : list
        List of dictionaries output from the ``parse_array_start`` method.
    """
    assert len(upstring) == len(component_list)

    def add_up_key(comp_dict, up_indicator):
        assert up_indicator == 'U' or up_indicator == "_"
        comp_dict['up'] = up_indicator == 'U'

    for comp_dict, up_indicator in zip(component_list, upstring):
        add_up_key(comp_dict, up_indicator)



"""
GaleraCnf - file ``/etc/my.cnf.d/galera.cnf``
=============================================

This module provides parsing for the galera configuration of
MySQL. The input is the contents of the file
`/etc/my.cnf.d/galera.cnf`.  Typical contents of the `galera.cnf`
file looks like this::

    [client]
    port = 3306
    socket = /var/lib/mysql/mysql.sock

    [isamchk]
    key_buffer_size = 16M

    [mysqld]
    basedir = /usr
    binlog_format = ROW
    datadir = /var/lib/mysql
    default-storage-engine = innodb
    expire_logs_days = 10
    innodb_autoinc_lock_mode = 2
    innodb_locks_unsafe_for_binlog = 1
    key_buffer_size = 16M
    log-error = /var/log/mariadb/mariadb.log
    max_allowed_packet = 16M
    max_binlog_size = 100M
    max_connections = 8192
    wsrep_max_ws_rows = 131072
    wsrep_max_ws_size = 1073741824

    [mysqld_safe]
    log-error = /var/log/mariadb/mariadb.log
    nice = 0
    socket = /var/lib/mysql/mysql.sock

    [mysqldump]
    max_allowed_packet = 16M
    quick
    quote-names

See the ``IniConfigFile`` base class for examples.

"""
from .. import parser, IniConfigFile
from insights.specs import Specs


@parser(Specs.galera_cnf)
class GaleraCnf(IniConfigFile):
    """Parses the content of `/etc/my.cnf.d/galera.cnf`."""

    def parse_content(self, content, allow_no_value=True):
        """Calls parent method to parse contents but overrides parameters.

        The galera config file may have keys with no value. This class
        implements ``parse_content`` in order to pass the flag
        ``allow_no_value`` to the parent parser in order to allow parsing
        of the no-value keys.
        """
        super(GaleraCnf, self).parse_content(content, allow_no_value=allow_no_value)



import pytest
import doctest

from insights.tests import context_wrap
from insights.parsers import SkipException, ParseException
from insights.parsers import package_provides
from insights.parsers.package_provides import PackageProvidesCommand

PACKAGE_COMMAND = """
/usr/bin/java java-11-openjdk-11.0.9.11-2.el8_3.x86_64
/usr/sbin/httpd httpd-2.4.22-7.el7.x86_64
/usr/lib/jvm/jre/bin/java java-1.8.0-openjdk-1.8.0.272.b10-3.el8_3.x86_64
/opt/rh/httpd24/root/usr/sbin/httpd httpd24-httpd-2.4.34-7.el7.x86_64
"""

PACKAGE_COMMAND_EMPTY = """
"""

PACKAGE_COMMAND_NOT_MATCH = """
jdk-9/bin/java file /root/jdk-9/bin/java is not owned by any package
bin/httpd file /root/bin/httpd is not owned by any package
"""


def test_package_provides_command():
    package = PackageProvidesCommand(context_wrap(PACKAGE_COMMAND))
    assert len(package.commands) == 4
    assert len(package.packages) == 4
    assert '/usr/bin/java' in package
    assert '/usr/sbin/httpd' in package.commands
    assert package['/usr/bin/java'] == 'java-11-openjdk-11.0.9.11-2.el8_3.x86_64'
    assert package['/opt/rh/httpd24/root/usr/sbin/httpd'] == 'httpd24-httpd-2.4.34-7.el7.x86_64'


def test_package_provides_command_AB():
    with pytest.raises(SkipException):
        PackageProvidesCommand(context_wrap(PACKAGE_COMMAND_EMPTY))

    with pytest.raises(ParseException):
        PackageProvidesCommand(context_wrap(PACKAGE_COMMAND_NOT_MATCH))


def test_doc_examples():
    env = {
        'cmd_package': PackageProvidesCommand(context_wrap(PACKAGE_COMMAND)),
    }
    failed, _ = doctest.testmod(package_provides, globs=env)
    assert failed == 0



import doctest
import pytest
from insights.parsers import cpupower_frequency_info, ParseException, SkipException
from insights.parsers.cpupower_frequency_info import CpupowerFrequencyInfo
from insights.tests import context_wrap

CPUPOWER_INFO = """
analyzing CPU 0:
  test:
    test_key1: value1
    test2:
      test_key2: value2
        test3:
          test_key3: value3
  driver: pcc-cpufreq
  CPUs which run at the same hardware frequency: 0
  CPUs which need to have their frequency coordinated by software: 0
  maximum transition latency:  Cannot determine or is not supported.
  hardware limits: 1.20 GHz - 2.20 GHz
  available cpufreq governors: conservative userspace powersave ondemand performance
  current policy: frequency should be within 1.20 GHz and 2.20 GHz.
                  The governor "performance" may decide which speed to use
                  within this range.
  current CPU frequency: 2.38 GHz (asserted by call to hardware)
  boost state support:
    Supported: yes
    Active: yes
    2700 MHz max turbo 4 active cores
    2800 MHz max turbo 3 active cores
    2900 MHz max turbo 2 active cores
    3000 MHz max turbo 1 active cores
""".strip()

CPUPOWER_INFO_MULTI = """
analyzing CPU 0:
  driver: intel_pstate
  CPUs which run at the same hardware frequency: 0
  CPUs which need to have their frequency coordinated by software: 0
  maximum transition latency:  Cannot determine or is not supported.
  hardware limits: 800 MHz - 3.00 GHz
  available cpufreq governors: performance powersave
  current policy: frequency should be within 800 MHz and 3.00 GHz.
                  The governor "performance" may decide which speed to use
                  within this range.
  current CPU frequency: Unable to call hardware
  current CPU frequency: 1.22 GHz (asserted by call to kernel)
  boost state support:
    Supported: yes
    Active: yes
    2700 MHz max turbo 4 active cores
    2800 MHz max turbo 3 active cores
    2900 MHz max turbo 2 active cores
    3000 MHz max turbo 1 active cores
analyzing CPU 1:
  driver: intel_pstate
  CPUs which run at the same hardware frequency: 1
  CPUs which need to have their frequency coordinated by software: 1
  maximum transition latency:  Cannot determine or is not supported.
  hardware limits: 800 MHz - 3.00 GHz
  available cpufreq governors: performance powersave
  current policy: frequency should be within 800 MHz and 3.00 GHz.
                  The governor "performance" may decide which speed to use
                  within this range.
  current CPU frequency: Unable to call hardware
  current CPU frequency: 1.22 GHz (asserted by call to kernel)
  boost state support:
    Supported: yes
    Active: yes
analyzing CPU 2:
  driver: intel_pstate
  CPUs which run at the same hardware frequency: 2
  CPUs which need to have their frequency coordinated by software: 2
  maximum transition latency:  Cannot determine or is not supported.
  hardware limits: 800 MHz - 3.00 GHz
  available cpufreq governors: performance powersave
  current policy: frequency should be within 800 MHz and 3.00 GHz.
                  The governor "performance" may decide which speed to use
                  within this range.
  current CPU frequency: Unable to call hardware
  current CPU frequency: 871 MHz (asserted by call to kernel)
  boost state support:
    Supported: yes
    Active: yes
analyzing CPU 3:
  driver: intel_pstate
  CPUs which run at the same hardware frequency: 3
  CPUs which need to have their frequency coordinated by software: 3
  maximum transition latency:  Cannot determine or is not supported.
  hardware limits: 800 MHz - 3.00 GHz
  available cpufreq governors: performance powersave
  current policy: frequency should be within 800 MHz and 3.00 GHz.
                  The governor "performance" may decide which speed to use
                  within this range.
  current CPU frequency: Unable to call hardware
  current CPU frequency: 868 MHz (asserted by call to kernel)
  boost state support:
    Supported: yes
    Active: yes
"""

CPUPOWER_INFO_INVALID = """
ERROR FIRST LINE:
  driver: pcc-cpufreq
  CPUs which run at the same hardware frequency: 0
  CPUs which need to have their frequency coordinated by software: 0
  maximum transition latency:  Cannot determine or is not supported.
  hardware limits: 1.20 GHz - 2.20 GHz
  available cpufreq governors: conservative userspace powersave ondemand performance
  current policy: frequency should be within 1.20 GHz and 2.20 GHz.
                  The governor "performance" may decide which speed to use
                  within this range.
  current CPU frequency: 2.38 GHz (asserted by call to hardware)
  boost state support:
    Supported: yes
    Active: yes
    2700 MHz max turbo 4 active cores
    2800 MHz max turbo 3 active cores
    2900 MHz max turbo 2 active cores
    3000 MHz max turbo 1 active cores
""".strip()

CPUPOWER_INFO_EMPTY = """
""".strip()


def test_cpupower_frequency_info():
    cpupower_info = CpupowerFrequencyInfo(context_wrap(CPUPOWER_INFO))
    assert cpupower_info['analyzing CPU 0']['boost state support']['Supported'] == 'yes'
    assert cpupower_info['analyzing CPU 0']['boost state support']['Active'] == 'yes'
    assert cpupower_info['analyzing CPU 0']['current policy'] == 'frequency should be within 1.20 GHz and 2.20 GHz. The governor "performance" may decide which speed to use within this range.'
    assert cpupower_info['analyzing CPU 0']['boost state support']['2700 MHz max turbo 4 active cores'] is True


def test_cpupower_frequency_info_multi():
    cpupower_info = CpupowerFrequencyInfo(context_wrap(CPUPOWER_INFO_MULTI))
    assert cpupower_info['analyzing CPU 3']['boost state support']['Supported'] == 'yes'
    assert cpupower_info['analyzing CPU 3']['boost state support']['Active'] == 'yes'
    assert cpupower_info['analyzing CPU 3']['current policy'] == 'frequency should be within 800 MHz and 3.00 GHz. The governor "performance" may decide which speed to use within this range.'


def test_invalid():
    with pytest.raises(ParseException) as e:
        CpupowerFrequencyInfo(context_wrap(CPUPOWER_INFO_INVALID))
    assert "Incorrect content" in str(e)


def test_empty():
    with pytest.raises(SkipException) as e:
        CpupowerFrequencyInfo(context_wrap(CPUPOWER_INFO_EMPTY))
    assert "Empty content" in str(e)


def test_cpupower_frequency_info_doc_examples():
    env = {
        'cpupower_frequency_info': CpupowerFrequencyInfo(
            context_wrap(CPUPOWER_INFO_MULTI)),
    }
    failed, total = doctest.testmod(cpupower_frequency_info, globs=env)
    assert failed == 0



from insights.parsers.nfs_exports import NFSExports, NFSExportsD
from insights.parsers.tests.test_nfs_exports import EXPORTS
from insights.combiners import nfs_exports
from insights.tests import context_wrap

from doctest import testmod

EXPORTS_D_EXTRAS = """
/mnt/work       *(rw,sync)
/mnt/backup     10.0.0.0/24(rw,sync,no_root_squash)
# This is a duplicate share but a new host, so it should be stored
/home/insights/shared/rw ins4.example.com(rw,sync,no_root_squash)
# We expect the following to be stored in the ignored_exports property
# as it has a duplicate host declaration for this path.
/home/insights/shared/rw ins1.example.com(rw,sync,no_root_squash)
/home/insights/shared/rw ins2.example.com(rw,sync,no_root_squash)
"""

# Set up combined test environment for docs and for more thorough functional
# testing.
nfs_exportsf = NFSExports(
    context_wrap(EXPORTS, path="/etc/exports")
)
nfs_exportsd = [NFSExportsD(
    context_wrap(EXPORTS_D_EXTRAS, path="/etc/exports.d/mnt.exports")
)]
combined = nfs_exports.AllNFSExports(nfs_exportsf, nfs_exportsd)


def test_nfs_exports_docs():
    failed, total = testmod(nfs_exports, globs={'all_nfs': combined})
    assert failed == 0


def test_nfs_export_combiner():
    assert combined
    assert hasattr(combined, 'exports')
    assert isinstance(combined.exports, dict)
    assert combined.exports
    assert len(combined.exports) == 7
    assert sorted(combined.exports.keys()) == sorted([
        '/home/utcs/shared/ro', '/home/insights/shared/rw',
        '/home/insights/shared/special/all/mail',
        '/home/insights/ins/special/all/config', '/home/example', '/mnt/work',
        '/mnt/backup'
    ])
    for path, hosts in nfs_exportsf.data.items():
        assert hosts == combined.exports[path]
    assert sorted(combined.exports['/home/insights/shared/rw'].keys()) == sorted([
        '@group', 'ins1.example.com', 'ins2.example.com', 'ins4.example.com'
    ])

    assert hasattr(combined, 'ignored_exports')
    assert isinstance(combined.ignored_exports, dict)
    assert combined.ignored_exports
    assert len(combined.ignored_exports) == 2
    assert sorted(combined.ignored_exports.keys()) == sorted([
        '/home/insights/shared/rw', '/home/example'
    ])

    assert hasattr(combined, 'raw_lines')
    assert isinstance(combined.raw_lines, dict)
    assert combined.raw_lines
    assert combined.raw_lines.keys() == combined.exports.keys()
    # Lines from only the exports file:
    assert combined.raw_lines['/home/utcs/shared/ro'] == {
        '/etc/exports': [
            '/home/utcs/shared/ro                    @group(ro,sync)   ins1.example.com(rw,sync,no_root_squash) ins2.example.com(rw,sync,no_root_squash)'
        ]
    }
    # Lines from only the exports.d file:
    assert combined.raw_lines['/mnt/backup'] == {
        '/etc/exports.d/mnt.exports': [
            '/mnt/backup     10.0.0.0/24(rw,sync,no_root_squash)'
        ],
    }
    # Lines from both the exports and exports.d files:
    assert combined.raw_lines['/home/insights/shared/rw'] == {
        '/etc/exports': [
            '/home/insights/shared/rw                @group(rw,sync)   ins1.example.com(rw,sync,no_root_squash) ins2.example.com(ro,sync,no_root_squash)',
        ],
        '/etc/exports.d/mnt.exports': [
            '/home/insights/shared/rw ins4.example.com(rw,sync,no_root_squash)',
            '/home/insights/shared/rw ins1.example.com(rw,sync,no_root_squash)',
            '/home/insights/shared/rw ins2.example.com(rw,sync,no_root_squash)'
        ]
    }


def test_nfs_exports_with_no_exports_d():
    combined = nfs_exports.AllNFSExports(nfs_exportsf, None)
    assert hasattr(combined, 'exports')
    assert isinstance(combined.exports, dict)
    assert combined.exports
    assert len(combined.exports) == 5
    assert sorted(combined.exports.keys()) == sorted([
        '/home/utcs/shared/ro', '/home/insights/shared/rw',
        '/home/insights/shared/special/all/mail',
        '/home/insights/ins/special/all/config', '/home/example'
    ])
    for path, hosts in nfs_exportsf.data.items():
        assert hosts == combined.exports[path]
    assert sorted(combined.exports['/home/insights/shared/rw'].keys()) == sorted([
        '@group', 'ins1.example.com', 'ins2.example.com', 'ins4.example.com'
    ])

    assert hasattr(combined, 'ignored_exports')
    assert isinstance(combined.ignored_exports, dict)
    assert combined.ignored_exports
    assert len(combined.ignored_exports) == 1
    assert sorted(combined.ignored_exports.keys()) == ['/home/example']

    assert hasattr(combined, 'raw_lines')
    assert isinstance(combined.raw_lines, dict)
    assert combined.raw_lines



from __future__ import print_function
from insights.parsers import lvm
from insights.tests import context_wrap
from .lvm_test_data import LVMCONFIG

WARNINGS_CONTENT = """
WARNING
valid data 1
Checksum Error
valid data 2
  Failed to write
  Attempt To Close Device
valid data 3
""".strip()

WARNINGS_FOUND = """
WARNING
Checksum Error
  Failed to write
  Attempt To Close Device
""".strip()

VGSHEADING_CONTENT = """
Configuration setting "activation/thin_check_executable" unknown.
Configuration setting "activation/thin_check_options" unknown.
Configuration setting "activation/thin_check_executable" unknown.
Configuration setting "activation/thin_check_options" unknown.
WARNING: Locking disabled. Be careful! This could corrupt your metadata.
  Using volume group(s) on command line.
Found duplicate PV qJMs5CEKY89qzq56E2vVeBvxzJGw2sA1: using /dev/mapper/mpathbp2 not /dev/cciss/c0d1p2
Using duplicate PV /dev/mapper/mpathbp2 from subsystem DM, ignoring /dev/cciss/c0d1p2
WARNING: Inconsistent metadata found for VG vgshared - updating to use version 26x
  Archiving volume group "vgshared" metadata (seqno 26).
  Archiving volume group "vgshared" metadata (seqno 27).
  Creating volume group backup "/etc/lvm/backup/vgshared" (seqno 27).
VG       Attr   Ext     #PV #LV #SN VSize   VFree   VG UUID                                VProfile #VMda VMdaFree  VMdaSize  #VMdaUse VG Tags
vg00     wz--n- 128.00m   1   7   0 279.00g   5.00g MiuKdK-ruw4-UG1i-0uLE-4dfB-bk2v-uxte2W              1   506.00k  1020.00k        1
vg00alt  wz--n-   4.00m   1   5   0 279.11g 265.11g LNzjbn-HU4C-WFCM-cRZ6-81ns-4rke-X8N0DA              1   507.00k  1020.00k        1
vg01     wz--n-   4.00m   2   1   0 199.99g 100.99g 7O9ePI-M1Kp-H9VH-Lxt3-pS1Z-nLQB-P1IIO3              2   508.00k  1020.00k        2
vgshared wz--nc   4.00m   6   1   0   1.76t 899.98g fFYYw3-Ns8Q-akPI-rrFU-YYn5-nKkk-UW67qO              6   507.00k  1020.00k        6
  Reloading config files
  Wiping internal VG cache
Configuration setting "activation/thin_check_executable" unknown.
Configuration setting "activation/thin_check_options" unknown.
""".strip()


def test_find_warnings():
    data = [l for l in lvm.find_warnings(WARNINGS_CONTENT.splitlines())]
    assert len(data) == len(WARNINGS_FOUND.splitlines())
    assert data == WARNINGS_FOUND.splitlines()


def compare_partial_dicts(result, expected):
    """
    Make sure all the keys in expected are matched by keys in result, and
    that the values stored in those keys match.  Result can contain more
    items than expected - those are ignored.

    Used in the test_lvs, test_pvs and test_vgs tests.
    """
    # return all(result[k] == expected[k] for k in expected.keys())
    mismatches = 0
    for k in expected.keys():
        if not result[k] == expected[k]:
            print("Failed for key {k}, {r} != {e}".format(k=k, r=result[k], e=expected[k]))
            mismatches += 1
    return mismatches == 0


def test_lvmconfig():
    p = lvm.LvmConfig(context_wrap(LVMCONFIG))
    assert p.data["dmeventd"]["raid_library"] == "libdevmapper-event-lvm2raid.so"
    assert p.data["global"]["thin_check_options"] == ["-q", "--clear-needs-check-flag"]


def test_vgsheading_warnings():
    result = lvm.VgsHeadings(context_wrap(VGSHEADING_CONTENT))
    assert len(result.warnings) == 6
    assert 'Configuration setting "activation/thin_check_executable" unknown.' in result.warnings
    assert 'WARNING: Locking disabled. Be careful! This could corrupt your metadata.' in result.warnings



"""
DockerStorageSetup - file ``/etc/sysconfig/docker-storage-setup``
=================================================================

A fairly simple parser to read the contents of the docker storage setup
configuration.

Sample input::

    # Edit this file to override any configuration options specified in
    # /usr/lib/docker-storage-setup/docker-storage-setup.
    #
    # For more details refer to "man docker-storage-setup"
    VG=vgtest
    AUTO_EXTEND_POOL=yes
    ##name = mydomain
    POOL_AUTOEXTEND_THRESHOLD=60
    POOL_AUTOEXTEND_PERCENT=20

Examples:

    >>> setup = shared[DockerStorageSetup]
    >>> setup['VG'] # Pseudo-dict access
    'vgtest'
    >>> 'name' in setup
    False
    >>> setup.data['POOL_AUTOEXTEND_THRESHOLD'] # old style access
    '60'

"""

from insights.util import deprecated
from .. import SysconfigOptions, parser
from insights.specs import Specs


@parser(Specs.docker_storage_setup)
class DockerStorageSetup(SysconfigOptions):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.sysconfig.DockerStorageSetupSysconfig` instead.

    A parser for accessing /etc/sysconfig/docker-storage-setup.
    """
    def __init__(self, *args, **kwargs):
        deprecated(DockerStorageSetup, "Import DockerStorageSetupSysconfig from insights.parsers.sysconfig instead")
        super(DockerStorageSetup, self).__init__(*args, **kwargs)



"""
ZiplConf - configuration file for zipl
======================================

A parser file for parsing and extracting data from ``/etc/zipl.conf`` file.

Sample input::

    [defaultboot]
    defaultauto
    prompt=1
    timeout=5
    default=linux
    target=/boot
    [linux]
        image=/boot/vmlinuz-3.10.0-693.el7.s390x
        ramdisk=/boot/initramfs-3.10.0-693.el7.s390x.img
        parameters="root=/dev/mapper/rhel_gss5-root crashkernel=auto rd.dasd=0.0.0100 rd.dasd=0.0.0101 rd.dasd=0.0.0102 rd.lvm.lv=rhel_gss5/root rd.lvm.lv=rhel_gss5/swap net.ifnames=0 rd.znet=qeth,0.0.0600,0.0.0601,0.0.0602,layer2=0,portname=gss5,portno=0 LANG=en_US.UTF-8"
    [linux-0-rescue-a27932c8d57248e390cee3798bbd3709]
        image=/boot/vmlinuz-0-rescue-a27932c8d57248e390cee3798bbd3709
        ramdisk=/boot/initramfs-0-rescue-a27932c8d57248e390cee3798bbd3709.img
        parameters="root=/dev/mapper/rhel_gss5-root crashkernel=auto rd.dasd=0.0.0100 rd.dasd=0.0.0101 rd.dasd=0.0.0102 rd.lvm.lv=rhel_gss5/root rd.lvm.lv=rhel_gss5/swap net.ifnames=0 rd.znet=qeth,0.0.0600,0.0.0601,0.0.0602,layer2=0,portname=gss5,portno=0"
    # Configuration for dumping to SCSI disk
    # Separate IPL and dump partitions
    [dumpscsi]
    target=/boot
    dumptofs=/dev/sda2
    parameters="dump_dir=/mydumps dump_compress=none dump_mode=auto"
    # Menu containing two DASD boot configurations
    :menu1
    1=linux
    2=linux-0-rescue-a27932c8d57248e390cee3798bbd3709
    default=1
    prompt=1
    timeout=30

This module contains one parser:

ZiplConf - file ``/etc/zipl.conf``
----------------------------------

Examples:
    >>> zipl_info['linux']['image']
    '/boot/vmlinuz-3.10.0-693.el7.s390x'
    >>> zipl_info.images
    {'linux':'/boot/vmlinuz-3.10.0-693.el7.s390x','linux-0-rescue-a27932c8d57248e390cee3798bbd3709':'/boot/vmlinuz-0-rescue-a27932c8d57248e390cee3798bbd3709'}
    >>> zipl_info.dumptofses
    {'dumpscsi':'/dev/sda2'}
    >>> zipl_info[':menu1']['1']
    'linux'
    >>> 'defaultauto' in zipl_info['global']
    True
    >>> zipl_info['global']['defaultauto']
    None
"""

from .. import parser, Parser, get_active_lines, LegacyItemAccess
from insights.parsers import ParseException
from insights.specs import Specs


@parser(Specs.zipl_conf)
class ZiplConf(LegacyItemAccess, Parser):
    """
    The zipl.conf file basically contains key-value pairs or single command based on the line.
    Section name is quoted with '[]' and menu name is started with ':'.

    Raises:
        ParseException: when the first active line is not a section
    """
    def __init__(self, *args, **kwargs):
        self._images = {}
        self._dumptofses = {}
        super(ZiplConf, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        self.data = {}
        lines = get_active_lines(content)
        line0 = lines[0]
        if line0.startswith('[') and line0.endswith(']'):
            current = line0[1:-1]
            self.data[current] = {}
            for line in lines[1:]:
                if line.startswith('[') and line.endswith(']'):
                    current = line[1:-1]
                    self.data[current] = {}
                elif line.startswith(':'):
                    current = line
                    self.data[current] = {}
                else:
                    if '=' in line:
                        k, v = [s.strip() for s in line.split('=', 1)]
                        if k == "image":
                            self._images[current] = v
                        elif k == "dumptofs":
                            self._dumptofses[current] = v
                        self.data[current][k] = v
                    else:
                        self.data[current][line] = True
        else:
            raise ParseException('Invalid zipl configuration file is found.')

    @property
    def images(self):
        """
        Get all `image` items referenced in zipl configuration file

        Returns:
            (dict): Returns a dict of the `section` and `image` names referenced
                    in zipl configuration file
        """
        return self._images

    @property
    def dumptofses(self):
        """
        Get all `dumptofs` items referenced in zipl configuration file

        Returns:
            (dict): Returns a dict of the `section` and `dumptofs` names referenced
                    in zipl configuration file
        """
        return self._dumptofses



from insights.tests import context_wrap
from insights.parsers import system_time


CHRONY_CONF = """
# Use public servers from the pool.ntp.org project.
# Please consider joining the pool (http://www.pool.ntp.org/join.html).
server 0.rhel.pool.ntp.org iburst
server 1.rhel.pool.ntp.org iburst
server 2.rhel.pool.ntp.org iburst
server 3.rhel.pool.ntp.org iburst

# Ignore stratum in source selection.
stratumweight 0

# Record the rate at which the system clock gains/losses time.
driftfile /var/lib/chrony/drift

# Enable kernel RTC synchronization.
rtcsync

# In first three updates step the system clock instead of slew
# if the adjustment is larger than 10 seconds.
makestep 10 3

# Allow NTP client access from local network.
#allow 192.168/16

# Listen for commands only on localhost.
bindcmdaddress 127.0.0.1
bindcmdaddress ::1

# Serve time even if not synchronized to any NTP server.
#local stratum 10

keyfile /etc/chrony.keys

# Specify the key used as password for chronyc.
commandkey 1

# Generate command key if missing.
generatecommandkey

# Disable logging of client accesses.
noclientlog

# Send a message to syslog if a clock adjustment is larger than 0.5 seconds.
logchange 0.5

logdir /var/log/chrony
#log measurements statistics tracking

leapsecmode slew
maxslewrate 1000
smoothtime 400 0.001 leaponly
"""

STANDARD_NTP_CONF = """
# For more information about this file, see the man pages
# ntp.conf(5), ntp_acc(5), ntp_auth(5), ntp_clock(5), ntp_misc(5), ntp_mon(5).

driftfile /var/lib/ntp/drift

# Permit time synchronization with our time source, but do not
# permit the source to query or modify the service on this system.
restrict default nomodify notrap nopeer noquery

# Permit all access over the loopback interface.  This could
# be tightened as well, but to do so would effect some of
# the administrative functions.
restrict 127.0.0.1
restrict ::1

# Hosts on local network are less restricted.
#restrict 192.168.1.0 mask 255.255.255.0 nomodify notrap

# Use public servers from the pool.ntp.org project.
# Please consider joining the pool (http://www.pool.ntp.org/join.html).
#server 0.rhel.pool.ntp.org iburst
#server 1.rhel.pool.ntp.org iburst
#server 2.rhel.pool.ntp.org iburst

#broadcast 192.168.1.255 autokey        # broadcast server
#broadcastclient                        # broadcast client
#broadcast 224.0.1.1 autokey            # multicast server
#multicastclient 224.0.1.1              # multicast client
#manycastserver 239.255.254.254         # manycast server
#manycastclient 239.255.254.254 autokey # manycast client

# Enable public key cryptography.
#crypto

includefile /etc/ntp/crypto/pw

# Key file containing the keys and key identifiers used when operating
# with symmetric key cryptography.
keys /etc/ntp/keys

# Specify the key identifiers which are trusted.
#trustedkey 4 8 42

# Specify the key identifier to use with the ntpdc utility.
#requestkey 8

# Specify the key identifier to use with the ntpq utility.
#controlkey 8

# Enable writing of statistics records.
#statistics clockstats cryptostats loopstats peerstats

# Disable the monitoring facility to prevent amplification attacks using ntpdc
# monlist command when default restrict does not include the noquery flag. See
# CVE-2013-5211 for more details.
# Note: Monitoring will not be disabled with the limited restriction flag.
#disable monitor

# Set a couple of single keyword options:
broadcastclient
iburst

### Added by IPA Installer ###
server 127.127.1.0
fudge 127.127.1.0 stratum 10

server 10.20.30.40
server 192.168.1.111

### Added by IPA Installer ###
peer ntp1.example.com
peer ntp2.example.com
peer ntp3.example.com

"""

ZERO_HOSTS_NTP_CONF = """
broadcastclient
"""

NTP_TINKER_CONF = """
driftfile /var/lib/ntp/ntp.drift
restrict default kod nomodify notrap nopeer noquery
restrict 127.0.0.1
server 192.168.17.62 iburst
tinker step 0.9
tinker step step
tinker step 0.4
"""


def test_chrony_conf():
    ntp_obj = system_time.ChronyConf(context_wrap(CHRONY_CONF))
    result = ntp_obj.data
    assert result['server'] == ["0.rhel.pool.ntp.org iburst",
            "1.rhel.pool.ntp.org iburst",
            "2.rhel.pool.ntp.org iburst",
            "3.rhel.pool.ntp.org iburst"]
    # keywords that are present but single word are present but equal None
    assert 'rtcsync' in result
    assert result.get('rtcsync') is None
    assert result['leapsecmode'] == ['slew']
    assert result['smoothtime'] == ['400 0.001 leaponly']

    # Test get_param method:
    # - nonexistent keyword
    assert ntp_obj.get_param('device') == [None]
    assert ntp_obj.get_param('device', 'embn0') == [None]
    assert ntp_obj.get_param('device', default='bad') == ['bad']
    assert ntp_obj.get_param('device', param='embn0', default='bad') == ['bad']
    assert ntp_obj.get_param('device', 'embn0', 'bad') == ['bad']
    # - simple keyword with no value - default and param ignored
    assert ntp_obj.get_param('noclientlog') == [None]
    assert ntp_obj.get_param('noclientlog', 'true') == [None]
    assert ntp_obj.get_param('noclientlog', default='yes') == [None]
    assert ntp_obj.get_param('noclientlog', param='true', default='yes') == [None]
    assert ntp_obj.get_param('noclientlog', 'true', 'yes') == [None]
    # - simple keyword with value - default and param ignored
    assert ntp_obj.get_param('commandkey') == ['1']
    assert ntp_obj.get_param('commandkey', '0') == ['1']
    assert ntp_obj.get_param('commandkey', default='2') == ['1']
    assert ntp_obj.get_param('commandkey', param='0', default='2') == ['1']
    assert ntp_obj.get_param('commandkey', '0', '2') == ['1']

    # Test get_last method:
    # - nonexistent keyword
    assert ntp_obj.get_last('device') is None
    assert ntp_obj.get_last('device', 'embn0') is None
    assert ntp_obj.get_last('device', default='bad') == 'bad'
    assert ntp_obj.get_last('device', param='embn0', default='bad') == 'bad'
    assert ntp_obj.get_last('device', 'embn0', 'bad') == 'bad'
    # - simple keyword with no value - default and param ignored
    assert ntp_obj.get_last('noclientlog') is None
    assert ntp_obj.get_last('noclientlog', 'true') is None
    assert ntp_obj.get_last('noclientlog', default='yes') is None  # present, so no default
    assert ntp_obj.get_last('noclientlog', param='true', default='yes') is None
    assert ntp_obj.get_last('noclientlog', 'true', 'yes') is None
    # - simple keyword with value - default and param ignored
    assert ntp_obj.get_last('commandkey') == '1'
    assert ntp_obj.get_last('commandkey', '0') == '1'
    assert ntp_obj.get_last('commandkey', default='2') == '1'
    assert ntp_obj.get_last('commandkey', param='0', default='2') == '1'
    assert ntp_obj.get_last('commandkey', '0', '2') == '1'
    # More testing in ntp_tinker_conf


LOCALTIME = "/etc/localtime: timezone data, version 2, 5 gmt time flags, \
5 std time flags, no leap seconds, 69 transition times, 5 abbreviation chars"

LOCALTIME_BAD = "/etc/localtime: file not found"


def test_localtime():
    result = system_time.LocalTime(context_wrap(LOCALTIME)).data
    assert result['name'] == "/etc/localtime"
    assert result['version'] == "2"
    assert result['gmt_time_flag'] == "5"
    assert result['std_time_flag'] == "5"
    assert result['leap_second'] == "no"
    assert result['transition_time'] == "69"
    assert result['abbreviation_char'] == "5"


def test_bad_localtime():
    result = system_time.LocalTime(context_wrap(LOCALTIME_BAD)).data
    assert result == {'name': '/etc/localtime'}


NTPTIME = """
ntp_gettime() returns code 0 (OK)
  time dbbc595d.1adbd720  Thu, Oct 27 2016 18:45:49.104, (.104917550),
  maximum error 263240 us, estimated error 102 us, TAI offset 0
ntp_adjtime() returns code 0 (OK)
  modes 0x0 (),
  offset 0.000 us, frequency 4.201 ppm, interval 1 s,
  maximum error 263240 us, estimated error 102 us,
  status 0x2011 (PLL,INS,NANO),
  time constant 2, precision 0.001 us, tolerance 500 ppm,
""".strip()

NTPTIME_STATUS = """
ntp_gettime() returns code 5 (ERROR)
  time ddb6c4ae.e3485000  Wed, Nov 15 2017 13:50:38.887, (.887822),
  maximum error 16000000 us, estimated error 16000000 us, TAI offset 0
ntp_adjtime() returns code 5 (ERROR)
  modes 0x0 (),
  offset 0.000 us, frequency 0.000 ppm, interval 1 s,
  maximum error 16000000 us, estimated error 16000000 us,
  status 0x50 (INS,UNSYNC),
  time constant 2, precision 1.000 us, tolerance 500 ppm,
""".strip()


def test_ntptime():
    result = system_time.NtpTime(context_wrap(NTPTIME)).data
    assert result['ntp_gettime'] == '0'
    assert result['ntp_adjtime'] == '0'
    assert result['status'] == '0x2011'
    assert result['flags'] == ['PLL', 'INS', 'NANO']

    assert result['timecode'] == 'dbbc595d.1adbd720'
    assert result['timestamp'] == 'Thu, Oct 27 2016 18:45:49.104, (.104917550)'
    assert result['maximum error'] == 263240
    assert result['estimated error'] == 102
    assert result['TAI offset'] == 0
    assert result['modes'] == 0x0
    assert result['offset'] == 0.000
    assert result['frequency'] == 4.201
    assert result['interval'] == 1
    assert result['time constant'] == 2
    assert result['precision'] == 0.001
    assert result['tolerance'] == 500


def test_ntptime_status():
    result = system_time.NtpTime(context_wrap(NTPTIME_STATUS)).data
    assert result['ntp_gettime'] == '5'
    assert result['ntp_adjtime'] == '5'
    assert result['status'] == '0x50'
    assert result['flags'] == ['INS', 'UNSYNC']


def test_standard_ntp_conf():
    ntp_obj = system_time.NTPConf(context_wrap(STANDARD_NTP_CONF))
    assert ntp_obj
    assert hasattr(ntp_obj, 'data')

    # Test configuration dictionary
    data = ntp_obj.data
    assert data == {
        'driftfile': ['/var/lib/ntp/drift'],
        'restrict': [
            'default nomodify notrap nopeer noquery',
            '127.0.0.1',
            '::1',
        ],
        'includefile': ['/etc/ntp/crypto/pw'],
        'keys': ['/etc/ntp/keys'],
        'broadcastclient': None,
        'iburst': None,
        'server': [
            '127.127.1.0',
            '10.20.30.40',
            '192.168.1.111',
        ],
        'fudge': ['127.127.1.0 stratum 10'],
        'peer': [
            'ntp1.example.com',
            'ntp2.example.com',
            'ntp3.example.com',
        ]
    }

    # Test other attributes
    assert hasattr(ntp_obj, 'servers')
    assert ntp_obj.servers == \
        ['10.20.30.40', '127.127.1.0', '192.168.1.111']
    assert hasattr(ntp_obj, 'peers')
    assert ntp_obj.peers == \
        ['ntp1.example.com', 'ntp2.example.com', 'ntp3.example.com']

    # Test get_last with parameter not found
    assert ntp_obj.get_last('tinker', 'panic') is None


def test_zero_hosts_ntp_conf():
    ntp_obj = system_time.NTPConf(context_wrap(ZERO_HOSTS_NTP_CONF))
    assert ntp_obj
    assert hasattr(ntp_obj, 'data')
    assert ntp_obj.data == {
        'broadcastclient': None
    }
    assert hasattr(ntp_obj, 'servers')
    assert ntp_obj.servers == []
    assert hasattr(ntp_obj, 'peers')
    assert ntp_obj.peers == []


def test_ntp_get_tinker():
    ntp_obj = system_time.NTPConf(context_wrap(NTP_TINKER_CONF))
    assert ntp_obj
    assert hasattr(ntp_obj, 'data')
    assert 'tinker' in ntp_obj.data
    # tinker defined but panic not one of its parameters
    assert ntp_obj.get_last('tinker', 'panic') is None
    # tinker step defined
    assert ntp_obj.get_last('tinker', 'step') == '0.4'

    # - keyword parameter value - param not found
    assert ntp_obj.get_last('tinker', 'panic') is None
    assert ntp_obj.get_last('tinker', default='1') == 'step 0.4'  # No param: just find last line
    assert ntp_obj.get_last('tinker', param='panic', default='1') == '1'
    assert ntp_obj.get_last('tinker', 'panic', '1') == '1'

    # - keyword parameter value - param found
    assert ntp_obj.get_last('tinker', 'step') == '0.4'
    assert ntp_obj.get_last('tinker', param='step', default='1') == '0.4'  # Value from config
    assert ntp_obj.get_last('tinker', 'step', '1') == '0.4'



import doctest

from insights.parsers import ls_var_spool_postfix_maildrop
from insights.parsers.ls_var_spool_postfix_maildrop import LsVarSpoolPostfixMaildrop
from insights.tests import context_wrap

LS_VAR_SPOOL_POSTFIX_MAILDROP = """
total 20
-rwxr--r--. 1 0 90 258 Jul 11 15:54 55D6821C286
-rwxr--r--. 1 0 90 282 Jul 11 15:54 5852121C284
-rwxr--r--. 1 0 90 258 Jul 11 15:54 9FFEC21C287
-rwxr--r--. 1 0 90 258 Jul 11 15:54 E9A4521C285
-rwxr--r--. 1 0 90 258 Jul 11 15:54 EA60F21C288
"""


def test_ls_var_spool_postfix_maildrop():
    ls_var_spool_postfix_maildrop = LsVarSpoolPostfixMaildrop(context_wrap(LS_VAR_SPOOL_POSTFIX_MAILDROP, path='nsights_commands/ls_-ln_.var.spool.postfix.maildrop'))
    assert ls_var_spool_postfix_maildrop.files_of('/var/spool/postfix/maildrop') == ['55D6821C286', '5852121C284', '9FFEC21C287', 'E9A4521C285', 'EA60F21C288']


def test_ls_var_spool_postfix_maildrop_doc_examples():
    env = {
        'LsVarSpoolPostfixMaildrop': LsVarSpoolPostfixMaildrop,
        'ls_var_spool_postfix_maildrop': LsVarSpoolPostfixMaildrop(context_wrap(LS_VAR_SPOOL_POSTFIX_MAILDROP, path='nsights_commands/ls_-ln_.var.spool.postfix.maildrop')),
    }
    failed, total = doctest.testmod(ls_var_spool_postfix_maildrop, globs=env)
    assert failed == 0



import pytest
import doctest

from datetime import datetime
from insights.tests import context_wrap
from insights.parsers import zdump_v, SkipException

NORMAL_OUTPUT = '''
/etc/localtime  -9223372036854689408 = NULL
/etc/localtime  Sun Mar 10 06:59:59 2019 UTC = Sun Mar 10 01:59:59 2019 EST isdst=0 gmtoff=-18000
/etc/localtime  Sun Mar 10 07:00:00 2019 UTC = Sun Mar 10 03:00:00 2019 EDT isdst=1 gmtoff=-14400
/etc/localtime  Sun Nov  3 05:59:59 2019 UTC = Sun Nov  3 01:59:59 2019 EDT isdst=1 gmtoff=-14400
/etc/localtime  Sun Nov  3 06:00:00 2019 UTC = Sun Nov  3 01:00:00 2019 EST isdst=0 gmtoff=-18000
/etc/localtime  Sun Mar 14 06:59:59 2038 UTC = Sun Mar 14 01:59:59 2038 EST isdst=0 gmtoff=-18000
/etc/localtime  Sun Mar 14 07:00:00 2038 UTC = Sun Mar 14 03:00:00 2038 EDT isdst=1 gmtoff=-14400
/etc/localtime  9223372036854775807 = NUL
'''.strip()

BAD_OUTPUT1 = ''

BAD_OUTPUT2 = '''
/etc/localtime  -9223372036854775808 = NULL
/etc/localtime  9223372036854689407 = NULL
'''.strip()

BAD_OUTPUT3 = '''
/etc/localtime  -9223372036854775808 = NULL
/etc/localtime  Sun Nov  3 06:00:00 2019 UTC = Sun Nov  3 01:00:00 2019 EST gmtoff=-18000
/etc/localtime  Sun Mar 14 06:59:59 2038 UTC = Sun Mar 14 01:59:59 2038 EST isdst=0
/etc/localtime  9223372036854689407 = NULL
'''.strip()


def test_doc_examples():
    env = {
        'zdump': zdump_v.ZdumpV(
            context_wrap(NORMAL_OUTPUT)),
    }
    failed, total = doctest.testmod(zdump_v, globs=env)
    assert failed == 0


def test_zdump_v():
    zdump = zdump_v.ZdumpV(context_wrap(NORMAL_OUTPUT))
    assert len(zdump) == 6
    assert zdump[0].get('utc_time') == datetime(2019, 3, 10, 6, 59, 59)
    assert zdump[0].get('utc_time_raw') == 'Sun Mar 10 06:59:59 2019 UTC'
    assert zdump[0].get('local_time') == datetime(2019, 3, 10, 1, 59, 59)
    assert zdump[0].get('local_time_raw') == 'Sun Mar 10 01:59:59 2019 EST'
    assert zdump[0].get('isdst') == 0
    assert zdump[0].get('gmtoff') == -18000
    assert zdump[1].get('utc_time') == datetime(2019, 3, 10, 7, 0, 0)
    assert zdump[1].get('utc_time_raw') == 'Sun Mar 10 07:00:00 2019 UTC'
    assert zdump[1].get('local_time') == datetime(2019, 3, 10, 3, 0, 0)
    assert zdump[1].get('local_time_raw') == 'Sun Mar 10 03:00:00 2019 EDT'
    assert zdump[1].get('isdst') == 1
    assert zdump[1].get('gmtoff') == -14400
    assert zdump[2].get('utc_time') == datetime(2019, 11, 3, 5, 59, 59)
    assert zdump[2].get('utc_time_raw') == 'Sun Nov  3 05:59:59 2019 UTC'
    assert zdump[2].get('local_time') == datetime(2019, 11, 3, 1, 59, 59)
    assert zdump[2].get('local_time_raw') == 'Sun Nov  3 01:59:59 2019 EDT'
    assert zdump[2].get('isdst') == 1
    assert zdump[2].get('gmtoff') == -14400
    assert zdump[3].get('utc_time') == datetime(2019, 11, 3, 6, 0, 0)
    assert zdump[3].get('utc_time_raw') == 'Sun Nov  3 06:00:00 2019 UTC'
    assert zdump[3].get('local_time') == datetime(2019, 11, 3, 1, 0, 0)
    assert zdump[3].get('local_time_raw') == 'Sun Nov  3 01:00:00 2019 EST'
    assert zdump[3].get('isdst') == 0
    assert zdump[3].get('gmtoff') == -18000
    assert zdump[4].get('utc_time') == datetime(2038, 3, 14, 6, 59, 59)
    assert zdump[4].get('utc_time_raw') == 'Sun Mar 14 06:59:59 2038 UTC'
    assert zdump[4].get('local_time') == datetime(2038, 3, 14, 1, 59, 59)
    assert zdump[4].get('local_time_raw') == 'Sun Mar 14 01:59:59 2038 EST'
    assert zdump[4].get('isdst') == 0
    assert zdump[4].get('gmtoff') == -18000
    assert zdump[5].get('utc_time') == datetime(2038, 3, 14, 7, 0, 0)
    assert zdump[5].get('utc_time_raw') == 'Sun Mar 14 07:00:00 2038 UTC'
    assert zdump[5].get('local_time') == datetime(2038, 3, 14, 3, 0, 0)
    assert zdump[5].get('local_time_raw') == 'Sun Mar 14 03:00:00 2038 EDT'
    assert zdump[5].get('isdst') == 1
    assert zdump[5].get('gmtoff') == -14400

    zdump = zdump_v.ZdumpV(context_wrap(BAD_OUTPUT2))
    assert len(zdump) == 0

    zdump = zdump_v.ZdumpV(context_wrap(BAD_OUTPUT3))
    assert len(zdump) == 1
    assert zdump[0].get('utc_time') == datetime(2038, 3, 14, 6, 59, 59)
    assert zdump[0].get('utc_time_raw') == 'Sun Mar 14 06:59:59 2038 UTC'
    assert zdump[0].get('local_time') == datetime(2038, 3, 14, 1, 59, 59)
    assert zdump[0].get('local_time_raw') == 'Sun Mar 14 01:59:59 2038 EST'
    assert zdump[0].get('isdst') == 0
    assert zdump[0].get('gmtoff') is None


def test_fail():
    with pytest.raises(SkipException) as e:
        zdump_v.ZdumpV(context_wrap(BAD_OUTPUT1))
    assert "No Data from command: /usr/sbin/zdump -v /etc/localtime -c 2019,2039" in str(e)



from insights.tests import context_wrap
from insights.parsers.sssd_conf import SSSD_Config

sssd_conf_cnt = """

[sssd]
config_file_version = 2

# Number of times services should attempt to reconnect in the
# event of a crash or restart before they give up
reconnection_retries = 3

# If a back end is particularly slow you can raise this timeout here
sbus_timeout = 30
services = nss, pam

# SSSD will not start if you do not configure any domains.
# Add new domain configurations as [domain/<NAME>] sections, and
# then add the list of domains (in the order you want them to be
# queried) to the "domains" attribute below and uncomment it.
# domains = LOCAL,LDAP
domains = example.com
debug_level = 9

[nss]
# The following prevents SSSD from searching for the root user/group in
# all domains (you can add here a comma-separated list of system accounts that
# are always going to be /etc/passwd users, or that you want to filter out).
filter_groups = root
filter_users = root
reconnection_retries = 3

[pam]
reconnection_retries = 3

[domain/example.com]
id_provider = ldap
lookup_family_order = ipv4_only
ldap_uri = ldap://ldap.example.com/
ldap_search_base = dc=example,dc=com
enumerate = False
hbase_directory= /home
create_homedir = True
override_homedir = /home/%u
auth_provider = krb5
krb5_server = kerberos.example.com
krb5_realm = EXAMPLE.COM
"""

sssd_conf_no_domains = """
[sssd]
debug_level = 5
"""

sssd_conf_blank_domains = """
[sssd]
debug_level = 5
domains =
"""


def test_sssd_conf():
    result = SSSD_Config(context_wrap(sssd_conf_cnt))
    assert 'sssd' in result
    assert 'domain/example.com' in result

    assert result.getint('pam', 'reconnection_retries') == 3

    assert ['example.com'] == result.domains

    domain = result.domain_config('example.com')
    assert type(domain) == dict
    assert domain['id_provider'] == 'ldap'

    absent_domain = result.domain_config('example.org')
    assert type(absent_domain) == dict
    assert absent_domain == {}


def test_sssd_conf_empty_domains():
    conf = SSSD_Config(context_wrap(sssd_conf_no_domains))
    assert conf.domains == []

    conf = SSSD_Config(context_wrap(sssd_conf_blank_domains))
    assert conf.domains == []



from insights.parsers.galera_cnf import GaleraCnf
from insights.tests import context_wrap

GALERA_CNF = """
[client]
port = 3306
socket = /var/lib/mysql/mysql.sock

[isamchk]
key_buffer_size = 16M

[mysqld]
basedir = /usr
binlog_format = ROW
datadir = /var/lib/mysql
default-storage-engine = innodb
expire_logs_days = 10
innodb_autoinc_lock_mode = 2
innodb_locks_unsafe_for_binlog = 1
key_buffer_size = 16M
log-error = /var/log/mariadb/mariadb.log
max_allowed_packet = 16M
max_binlog_size = 100M
max_connections = 8192
wsrep_max_ws_rows = 131072
wsrep_max_ws_size = 1073741824

[mysqld_safe]
log-error = /var/log/mariadb/mariadb.log
nice = 0
socket = /var/lib/mysql/mysql.sock

[mysqldump]
max_allowed_packet = 16M
quick
quote-names
"""


def test_galera_cnf():
    cnf = GaleraCnf(context_wrap(GALERA_CNF))
    assert cnf is not None
    assert cnf.get('client', 'port') == '3306'
    assert cnf.get('isamchk', 'key_buffer_size') == '16M'
    assert cnf.get('mysqld', 'max_connections') == '8192'
    assert 'not_there' not in cnf
    assert cnf.get('mysqldump', 'quick') is None



import doctest
from insights.parsers import sysctl
from insights.tests import context_wrap
from insights.util import keys_in

SYSCTL_TEST = """
a=1
b = 2
c = include an = sign
""".strip()

SYSCTL_DOC_TEST = """
kernel.domainname = example.com
kernel.modprobe = /sbin/modprobe
""".strip()

SYSCTL_CONF_TEST = """
# sysctl.conf sample
#
  kernel.domainname = example.com
# kernel.domainname.invalid = notvalid.com

; this one has a space which will be written to the sysctl!
  kernel.modprobe = /sbin/mod probe
""".strip()

SYSCTL_CONF_INITRAMFS_TEST = """
initramfs:/etc/sysctl.conf
========================================================================
# sysctl settings are defined through files in
# /usr/lib/sysctl.d/, /run/sysctl.d/, and /etc/sysctl.d/.
#
# Vendors settings live in /usr/lib/sysctl.d/.
# To override a whole file, create a new file with the same in
# /etc/sysctl.d/ and put new settings there. To override
# only specific settings, add a file with a lexically later
# name in /etc/sysctl.d/ and put new settings there.
#
# For more information, see sysctl.conf(5) and sysctl.d(5).
fs.inotify.max_user_watches=524288
key2=value2
key2_alt=value2_alt
#  key3=value3
; key4=value4
========================================================================

initramfs:/etc/sysctl.d/*.conf
========================================================================
========================================================================
""".strip()


def test_sysctl():
    r = sysctl.Sysctl(context_wrap(SYSCTL_TEST))
    assert keys_in(["a", "b", "c"], r.data)
    assert r.data["a"] == "1"
    assert r.data["b"] == "2"
    assert r.data["c"] == "include an = sign"


def test_sysctl_conf():
    r = sysctl.SysctlConf(context_wrap(SYSCTL_CONF_TEST))
    assert keys_in(['kernel.domainname', 'kernel.modprobe'], r.data)
    assert r.data['kernel.domainname'] == 'example.com'
    assert r.data['kernel.modprobe'] == '/sbin/mod probe'
    assert 'kernel.domainname.invalid' not in r.data


def test_sysctl_conf_initramfs():
    r = sysctl.SysctlConfInitramfs(context_wrap(SYSCTL_CONF_INITRAMFS_TEST))
    assert r is not None
    assert r.get('max_user_watches') == [{'raw_message': 'fs.inotify.max_user_watches=524288'}]
    assert r.get('key2') == [{'raw_message': 'key2=value2'}, {'raw_message': 'key2_alt=value2_alt'}]
    assert r.get('key3') == []
    assert r.get('key4') == []


def test_docs():
    env = {
        'sysctl': sysctl.Sysctl(context_wrap(SYSCTL_DOC_TEST)),
        'sysctl_conf': sysctl.SysctlConf(context_wrap(SYSCTL_CONF_TEST)),
        'sysctl_initramfs': sysctl.SysctlConfInitramfs(context_wrap(SYSCTL_CONF_INITRAMFS_TEST))
    }
    failed, total = doctest.testmod(sysctl, globs=env)
    assert failed == 0



"""
LsSysFirmware - command ``ls /sys/firmware``
============================================

The ``ls -lanR /sys/firmware`` command provides information for the listing of
the ``/sys/firmware`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

Examples:
    >>> LS_SYS_FIRMWARE = '''
    ... /sys/firmware:
    ... total 0
    ... drwxr-xr-x.  5 0 0 0 Dec 22 17:56 .
    ... dr-xr-xr-x. 13 0 0 0 Dec 22 17:56 ..
    ... drwxr-xr-x.  5 0 0 0 Dec 22 17:56 acpi
    ... drwxr-xr-x.  3 0 0 0 Dec 22 17:57 dmi
    ... drwxr-xr-x. 10 0 0 0 Dec 22 17:57 memmap
    ...
    ... /sys/firmware/acpi:
    ... total 0
    ... drwxr-xr-x. 5 0 0    0 Dec 22 17:56 .
    ... drwxr-xr-x. 5 0 0    0 Dec 22 17:56 ..
    ... drwxr-xr-x. 6 0 0    0 Feb 10 15:54 hotplug
    ... drwxr-xr-x. 2 0 0    0 Feb 10 15:54 interrupts
    ... -r--r--r--. 1 0 0 4096 Feb 10 15:54 pm_profile
    ... drwxr-xr-x. 3 0 0    0 Dec 22 17:56 tables
    ... '''
    >>> ls_sys_firmware = LsSysFirmware(context_wrap(LS_SYS_FIRMWARE))
    >>> "acpi" in ls_sys_firmware
    False
    >>> "/sys/firmware/acpi" in ls_sys_firmware
    True
    >>> ls_sys_firmware.dirs_of("/sys/firmware")
    ['.', '..', 'acpi', 'dmi', 'memmap']
    >>> ls_sys_firmware.files_of("/sys/firmware/acpi")
    ['pm_profile']
"""
from .. import parser
from .. import FileListing, CommandParser
from insights.specs import Specs


@parser(Specs.ls_sys_firmware)
class LsSysFirmware(CommandParser, FileListing):
    """Parses output of ``ls -lanR /sys/firmware`` command."""
    pass



"""
EngineDBQuery - command ``engine-db-query --statement "<DB_QUERY>" --json``
============================================================================

Parses the output of the command `engine-db-query` returned in JSON format.
"""
from insights.core import CommandParser, JSONParser
from insights.parsers import SkipException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.engine_db_query_vdsm_version)
class EngineDBQueryVDSMversion(CommandParser, JSONParser):
    """
    Get the hostname & vdsm package version along with host info.

    Class for parsing the output of the command - ``engine-db-query --statement "SELECT vs.vds_name, rpm_version FROM vds_dynamic vd, vds_static vs WHERE vd.vds_id = vs.vds_id" --json``.

    Attributes:
        data (dict): Host info.

    Sample output of this command is::

        {
          "id_host": "None",
          "when": "2020-06-21 12:45:59",
          "time": "0.00263094902039",
          "name": "None",
          "description": "None",
          "type": "None",
          "kb": "None",
          "bugzilla": "None",
          "file": "",
          "path": "None",
          "id": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
          "hash": "d41d8cd98f00b204e9800998ecf8427e",
          "result": [{"vds_name": "hosto", "rpm_version": "vdsm-4.30.40-1.el7ev"}]
        }


    Examples:
        >>> output.get('id', None) == 'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
        True
        >>> output.result == [{'vds_name': 'hosto', 'rpm_version': 'vdsm-4.30.40-1.el7ev'}]
        True
    """
    def parse_content(self, content):
        if not content:
            raise SkipException("Empty output.")

        super(EngineDBQueryVDSMversion, self).parse_content(content)

    @property
    def result(self):
        """Get the value of 'result'."""
        return self.data.get('result', [])



"""
Postconf - command ``postconf``
===============================

PostconfBuiltin - command ``postconf -C builtin``
=================================================
"""

from insights.core import CommandParser
from insights.core.plugins import parser
from insights.specs import Specs
from insights.parsers import SkipException


class _Postconf(CommandParser, dict):
    """
    Class for parsing the ``postconf`` command.
    Sample input::

        smtpd_tls_loglevel = 0
        smtpd_tls_mandatory_ciphers = medium
        smtpd_tls_mandatory_exclude_ciphers =
        smtpd_tls_mandatory_protocols = !SSLv2, !SSLv3, !TLSv1

    Examples:
        >>> type(_postconf)
        <class 'insights.parsers.postconf._Postconf'>
        >>> _postconf['smtpd_tls_loglevel'] == '0'
        True
        >>> _postconf['smtpd_tls_mandatory_ciphers'] == 'medium'
        True
        >>> _postconf['smtpd_tls_mandatory_exclude_ciphers'] == ''
        True
        >>> _postconf['smtpd_tls_mandatory_protocols'] == '!SSLv2, !SSLv3, !TLSv1'
        True
    """

    def parse_content(self, content):
        if not content:
            raise SkipException

        data = dict()
        for line in content:
            if '=' in line:
                key, value = [i.strip() for i in line.split('=', 1)]
                data[key] = value

        if not data:
            raise SkipException

        self.update(data)


@parser(Specs.postconf_builtin)
class PostconfBuiltin(_Postconf):
    """
    Class for parsing the ``postconf -C builtin`` command.
    Sample input::

        smtpd_tls_loglevel = 0
        smtpd_tls_mandatory_ciphers = medium
        smtpd_tls_mandatory_exclude_ciphers =
        smtpd_tls_mandatory_protocols = !SSLv2, !SSLv3, !TLSv1

    Examples:
        >>> type(postconfb)
        <class 'insights.parsers.postconf.PostconfBuiltin'>
        >>> postconfb['smtpd_tls_loglevel'] == '0'
        True
        >>> postconfb['smtpd_tls_mandatory_ciphers'] == 'medium'
        True
        >>> postconfb['smtpd_tls_mandatory_exclude_ciphers'] == ''
        True
        >>> postconfb['smtpd_tls_mandatory_protocols'] == '!SSLv2, !SSLv3, !TLSv1'
        True
    """


@parser(Specs.postconf)
class Postconf(_Postconf):
    """
    Class for parsing the ``postconf`` command.
    Sample input::

        smtpd_tls_loglevel = 0
        smtpd_tls_mandatory_ciphers = medium
        smtpd_tls_mandatory_exclude_ciphers =
        smtpd_tls_mandatory_protocols = !SSLv2, !SSLv3, !TLSv1

    Examples:
        >>> type(postconf)
        <class 'insights.parsers.postconf.Postconf'>
        >>> postconf['smtpd_tls_loglevel'] == '0'
        True
        >>> postconf['smtpd_tls_mandatory_ciphers'] == 'medium'
        True
        >>> postconf['smtpd_tls_mandatory_exclude_ciphers'] == ''
        True
        >>> postconf['smtpd_tls_mandatory_protocols'] == '!SSLv2, !SSLv3, !TLSv1'
        True
    """



from insights.parsers.ld_library_path import PidLdLibraryPath
from insights.tests import context_wrap
from insights.parsers import ld_library_path, SkipException, ParseException
import doctest
import pytest

LD_LIBRARY_PATH_EMPTY = """
""".strip()

LD_LIBRARY_PATH_INVALID = """
LD_LIBRARY_PATH: Undefined variable.
""".strip()

LD_LIBRARY_PATH_DOC = """
105901 /usr/sap/RH1/SYS/exe/run:/usr/sap/RH1/SYS/exe/uc/linuxx86_64:/sapdb/clients/RH1/lib
105902 /usr/sap/RH1/SYS/exe/uc/linuxx86_64:/usr/sap/RH1/SYS/exe/run
""".strip()

LD_LIBRARY_PATH = """
105901 /usr/sap/RH1/SYS/exe/run:/usr/sap/RH1/SYS/exe/uc/linuxx86_64:/sapdb/clients/RH1/lib
105902 "/usr/sap/RH1/SYS/exe/uc/linuxx86_64:/usr/sap/RH1/SYS/exe/run"
105903 
105904 ''
""".strip()  # noqa: W391


def test_ld_library_path():
    ret = PidLdLibraryPath(context_wrap(LD_LIBRARY_PATH))
    assert len(ret) == 4
    assert ret[0].pid == '105901'
    assert ret[1].pid == '105902'
    assert ret[2].pid == '105903'
    assert ret[1].raw == LD_LIBRARY_PATH.splitlines()[1].split()[-1]
    assert ret[2].raw == ''
    assert ret[3].raw == "''"
    assert ret[2].path == ['']
    assert ret[3].path == ['']
    for p in LD_LIBRARY_PATH.splitlines()[0].split()[-1].split(':'):
        assert p in ret[0].path
    for p in LD_LIBRARY_PATH.splitlines()[1].split()[-1].strip('"').split(':'):
        assert p in ret[1].path


def test_empty_and_invalid():
    with pytest.raises(SkipException):
        PidLdLibraryPath(context_wrap(LD_LIBRARY_PATH_EMPTY))

    with pytest.raises(ParseException):
        PidLdLibraryPath(context_wrap(LD_LIBRARY_PATH_INVALID))


def test_doc_examples():
    env = {
        'ld_lib_path': PidLdLibraryPath(context_wrap(LD_LIBRARY_PATH_DOC)),
    }
    failed, total = doctest.testmod(ld_library_path, globs=env)
    assert failed == 0



from insights.core import YAMLParser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.branch_info)
class BranchInfo(YAMLParser):
    """ Class for parsing the content of ``branch_info``."""
    pass



from insights.util.canonical_facts import _filter_falsy


def test_identity():
    assert {"foo": "bar"} == _filter_falsy({"foo": "bar"})


def test_drops_none():
    assert {"foo": "bar"} == _filter_falsy({"foo": "bar", "baz": None})


def test_drops_empty_list():
    assert {"foo": "bar"} == _filter_falsy({"foo": "bar", "baz": []})
